#include "color.h"

//The bit array for each node in a color set

NodeBitArray::NodeBitArray(UINT id)
{
	nodeid=id;
	p_begin=new BitArrayPart;
	p_last=p_begin;
	numPart=1;
}
NodeBitArray::~NodeBitArray()
{
	BitArrayPart* p;
	do{
		p=p_begin;
		p_begin=p_begin->p_next;
		delete p;
		p=NULL;
	}while(p_begin!=NULL);
	numPart=0;
	p_last=NULL;
	p_begin=NULL;
}
void NodeBitArray::resetAll()
{
	BitArrayPart* p;
	for(p=p_begin; p!=NULL; p=p->p_next)//loop through every part
		for(int i=0; i<BIT_ARRAY_PART_SIZE; i++)//loop through every UINT
			(p->part)[i]=0;
}
 
bool NodeBitArray::equal(NodeBitArray& another)
{
	BitArrayPart* p1, *p2;
	for(p1=p_begin, p2=another.p_begin; p1!=NULL&&p2!=NULL; p1=p1->p_next, p2=p2->p_next)//loop through every part
		for(int i=0; i<BIT_ARRAY_PART_SIZE; i++)//loop through every UINT
			if( (p1->part)[i] != (p2->part)[i])
				return false;
	for(; p2!=NULL; p2=p2->p_next)
		for(int i=0; i<BIT_ARRAY_PART_SIZE; i++)
			if((p2->part)[i]!=0)
				return false;
	for(; p1!=NULL; p1=p1->p_next)
		for(int i=0; i<BIT_ARRAY_PART_SIZE; i++)
			if((p1->part)[i]!=0)
				return false;
	return true;
}
void NodeBitArray::setBitOn(int pos)
{
	//find which part the bit is in and where in this part
	int bigstart=pos/NBITS_ARRAY_PART;
	int bigoffset=pos%NBITS_ARRAY_PART;
	
	//find out which UINT it is in and where in this UINT
	int smallstart=bigoffset/NBITS_UINT;
	int smalloffset=bigoffset%NBITS_UINT;
	
	BitArrayPart* p;
	
	if(bigstart<numPart-1)
	{
		p=p_begin;
		for(int i=0; i<bigstart; i++)
			p=p->p_next;
	}else
	{
		//if needs to add new part, do it
		for(int i=numPart; i<=bigstart; i++)
		{
			p_last->p_next=new BitArrayPart;
			p_last=p_last->p_next;
			numPart++;
		}
		p=p_last;
	}
	
	setBitOnHelp((p->part)[smallstart], smalloffset);
}
void NodeBitArray::setBitOff(int pos)
{
	//find which part the bit is in and where in this part
	int bigstart=pos/NBITS_ARRAY_PART;
	int bigoffset=pos%NBITS_ARRAY_PART;
	
	//if the bit array is not long enough, then default the postion is 0
	if(bigstart>=numPart)
		return;
	
	//find out which UINT it is in and where in this UINT
	int smallstart=bigoffset/NBITS_UINT;
	int smalloffset=bigoffset%NBITS_UINT;
	
	//find the part
	BitArrayPart* p;
	if(bigstart<numPart-1)
	{
		p=p_begin;
		for(int i=0; i<bigstart; i++)
			p=p->p_next;
	}else
		p=p_last;
	
	setBitOffHelp( (p->part)[smallstart], smalloffset);
}
void NodeBitArray::setBitOnHelp(UINT& unit, int offset)
{
	UINT mask=1;
	mask<<=(NBITS_UINT-1-offset);
    unit|=mask;
}

void NodeBitArray::setBitOffHelp(UINT& unit, int offset)
{
	UINT mask=1;
	mask<<=(NBITS_UINT-1-offset);
	mask=~mask;
    unit&=mask;
}
//if a bit postion is 1, then increment the correponding frequency
//assume that the size of frequencies are big enough
void NodeBitArray::addBits(hash_map<UINT, int>& counts)
{
	BitArrayPart* p;
	int smallstart=0;
	
	for(p=p_begin; p!=NULL; p=p->p_next)//loop through every part
	{
		for(int i=0; i<BIT_ARRAY_PART_SIZE; i++)//loop through every UINT
		{
			UINT mask=1;
			mask<<=(NBITS_UINT-1);
			for(int offset=0; offset<NBITS_UINT; offset++)//loop through every bit
			{
               			if(mask & (p->part)[i])
                		{
                			hash_map<UINT, int>::iterator it;
                			it=counts.find(smallstart*NBITS_UINT+offset);
                			if(it==counts.end())
                				it=(counts.insert(hash_map<UINT, int>::value_type(smallstart*NBITS_UINT+offset,0))).first;
                			it->second++;
                		}
                		mask>>=1;
			}
			smallstart++;
		}
	}
}

bool NodeBitArray::checkBit(int pos)
{
	int bigstart=pos/NBITS_ARRAY_PART;
	int bigoffset=pos%NBITS_ARRAY_PART;
	int smallstart=bigoffset/NBITS_UINT;
	int smalloffset=bigoffset%NBITS_UINT;
	
	//find the part
	BitArrayPart* p;
	if(bigstart>numPart-1)
		return false;
	else if(bigstart==numPart-1)
		p=p_last;
	else
	{
		p=p_begin;
		for(int i=0; i<bigstart; i++)
			p=p->p_next;
	}
		
	UINT mask=1;
	mask<<=(NBITS_UINT-1-smalloffset);
	return (mask & (p->part)[smallstart]);
}

////////////////////////////////////////////////////////

void ColorBase::setParticipates(UINT color, int count)
{
	participates[color]=count;
}

int ColorBase::getParticipates(UINT color)
{
	hash_map<UINT, int>::iterator p;
	p=participates.find(color);
	if(p!=participates.end())
		return p->second;
	else
		return 0;
}

//////////////////////////////////////////////////////

void SubColor::mergeIn(SubColor* subcolor2)
{
	//for the neighbors of c2, 
	hash_map<UINT, int>::iterator pp2, pp1;
	
	for(pp2=subcolor2->participates.begin(); pp2!=subcolor2->participates.end(); pp2++)
	{
		UINT color=pp2->first;
		
		int pt2=pp2->second;
		if(pt2==0) continue;
		
		int pt1=getParticipates(color);
		setParticipates(color, pt1+pt2);
	}
	size+=subcolor2->size;
}
int SubColor::getSize()
{
	return size;
}

////////////////////////////////////////////////////


Color::Color(int inicolor, int cid, int attrV)
{
	if(cid>maxColorid)
		maxColorid=cid;
	colorid=cid;
	attrValues=attrV;
	//maxGap=0;
	//gapColor=-1;
	initialColor=inicolor;
}

Color::~Color()
{
	NodeMap::iterator p;
	for(p=nodeMap.begin(); p!=nodeMap.end(); p++)
	{	
		delete p->second;
		p->second=NULL;
	}
}

int Color::getSize()
{
	return nodeMap.size();
}

//update the frequency array of this color
void Color::updateParticipates()
{
	NodeMap::iterator p;
	participates.clear();
	//for each node increment the frequency array
	for(p=nodeMap.begin(); p!=nodeMap.end(); p++)
		p->second->addBits(participates);
}

//only update frequencies after a ocolorid splits into ocolorid and ncolorid
void Color::updateParticipates(UINT ocolorid, UINT ncolorid)
{
	int ocount=0;
	int ncount=0;
	
	NodeMap::iterator p;
	//for each node increment the frequency array
	for(p=nodeMap.begin(); p!=nodeMap.end(); p++)
	{
		if(p->second->checkBit(ocolorid))
			ocount++;
		if(p->second->checkBit(ncolorid))
			ncount++;
	}
	setParticipates(ocolorid, ocount);
	setParticipates(ncolorid, ncount);
}

//merge the bit postition c1 and c2 to c1
void Color::mergeNodeMap(UINT c1, UINT c2)
{
	int count=0;
	NodeMap::iterator p;
	//for each node increment the frequency array
	for(p=nodeMap.begin(); p!=nodeMap.end(); p++)
	{
		if(p->second->checkBit(c2))
		{
			p->second->setBitOff(c2);
			p->second->setBitOn(c1);
			count++;
		}else if(p->second->checkBit(c1))
			count++;
	}
	setParticipates(c1, count);
	setParticipates(c2, 0);
}

//only update participates for one particular colorid
void Color::updateParticipates(UINT colorid)
{
	int count=0;
	NodeMap::iterator p;
	//for each node increment the frequency array
	for(p=nodeMap.begin(); p!=nodeMap.end(); p++)
	{
		if(p->second->checkBit(colorid))
			count++;
	}
	setParticipates(colorid, count);
}

void Color::print()
{
	cout<<"--------------------------------------\n";
	cout<<"colorID: "<<colorid<<endl;
	cout<<"#Nodes: "<<nodeMap.size()<<endl;
}

int Color::maxColorid;
int Color::gid;
string Color::edgeType;
string Color::nodeAttr;

//for directed graph only
GroupDistance GroupDist(ColorBase* color1, ColorBase* color2, UINT id1, UINT id2, ColorArray* p_colorArr)
{
	
	GroupDistance gd;
	gd.minGroupSize=min(color1->getSize(), color2->getSize());
	
	float totalPart=0;
	gd.freqDist=0;
	gd.minusNumAgree=0;
	
	//make sure that part1 is always shorter than the part2
	hash_map<UINT, int> temp(color2->participates);	

	hash_map<UINT, int>::iterator p;

	for(p=color1->participates.begin(); p!=color1->participates.end(); p++)
	{	

		UINT color=p->first;

		int count1=p->second;
		int count2=color2->getParticipates(p->first);
		//cout<<"in\n";
		//cout<<"color:"<<p->first<<endl;
		//int count3=(*p_colorArr)[color]->getParticipates(id1);
		//cout<<"out\n";
		//int count4=(*p_colorArr)[color]->getParticipates(id2);
		if(count1>0 || count2>0)
		{
			float freq1=(float)count1/(float)color1->getSize();
			float freq2=(float)count2/(float)color2->getSize();		
			gd.freqDist+=fabs(freq1-freq2);
			totalPart++;
			if(freq1<0.5 && freq2<0.5)
				gd.minusNumAgree--;
			else if(freq1>0.5 && freq2>0.5)
				gd.minusNumAgree--;
			
		}
		if(count2>0)
			temp.erase(color);
		
	}

	for(p=temp.begin(); p!=temp.end(); p++)
	{
		UINT color=p->first;
		int count2=p->second;
		if(count2==0)
			continue;
		
		float freq2=(float)count2/(float)color2->getSize();		
		gd.freqDist+=freq2;
		totalPart++;
		if(freq2<0.5)
			gd.minusNumAgree--;
	}

	if(totalPart>0)
	{	gd.freqDist=(gd.freqDist/totalPart);
		if(gd.freqDist < 0)
			sleep(10);
	}	
	else
		gd.freqDist=1;
	return gd;
}
