#ifndef COLOR_H
#define COLOR_H
//The data structure used in the graph summarization technique
#include <vector>
#include <hash_map.h>
#include <assert.h>
#include <fstream>
#include "datastruct.h"
#include <math.h>

//#define DETAIL
using namespace std;
const int BIT_ARRAY_PART_SIZE=4;
const int NBITS_UINT=8*sizeof(UINT);
const int NBITS_ARRAY_PART=NBITS_UINT*BIT_ARRAY_PART_SIZE;
//the Bit Array of a node is allocated Part by part, 
//each part has BIT_ARRAY_PART_SIZE UINTs
struct BitArrayPart
{
	BitArrayPart()
	{
		for(int i=0; i<BIT_ARRAY_PART_SIZE; i++)
			part[i]=0;
		p_next=NULL;
	}
	UINT part[BIT_ARRAY_PART_SIZE];
	BitArrayPart* p_next;
};


//The bit array for each node in a color set
class NodeBitArray
{
public:
	NodeBitArray(UINT id);
	~NodeBitArray();
	void resetAll();
	 
	bool equal(NodeBitArray& another);
	void setBitOn(int pos);
	void setBitOff(int pos);
	void setBitOnHelp(UINT& unit, int offset);
	
	void setBitOffHelp(UINT& unit, int offset);
	//if a bit postion is 1, then increment the correponding frequency
	//assume that the size of frequencies are big enough
	void addBits(hash_map<UINT, int>& counts);
	bool checkBit(int pos);
private:
	UINT nodeid;
	BitArrayPart* p_begin;//the first Part
	BitArrayPart* p_last;//the last part, for the convinience of appending
							//a new part at the back of the list
	int numPart;
};

typedef hash_map<UINT, NodeBitArray*> NodeMap;

class ColorBase
{
public:
	hash_map<UINT, int> participates;
	void setParticipates(UINT color, int count);
	
	int getParticipates(UINT color);
	virtual int getSize()=0;
};

class SubColor: public ColorBase
{
public:
	int size;
	void mergeIn(SubColor* subcolor2);
	int getSize();
};

class Color: public ColorBase
{
public:
	static int maxColorid;
	static int gid;
	static string edgeType;
	static string nodeAttr;

	//vector<float> frequencies;
	//float maxGap;//the max randomness 
	//int gapColor;//the neighbor color that has the maxGap
	NodeMap nodeMap;
	int colorid;
	int initialColor;
	int attrValues;
	Color(int inicolor, int cid, int attrV);
	
	~Color();
	
	int getSize();

	//update the frequency array of this color
	void updateParticipates();
	
	//only update frequencies after a ocolorid splits into ocolorid and ncolorid
	void updateParticipates(UINT ocolorid, UINT ncolorid);
	
	//merge the bit postition c1 and c2 to c1
	void mergeNodeMap(UINT c1, UINT c2);
	
	//only update participates for one particular colorid
	void updateParticipates(UINT colorid);
	
	void print();
	
};
typedef hash_map<UINT, Color*> ColorArray;

GroupDistance GroupDist(ColorBase* color1, ColorBase* color2, UINT id1, UINT id2, ColorArray* p_colorArr);

#endif
