#include "merger.h"

void printColorSet(ColorSet& cs)
{
	ColorSet::iterator p;
	cout<<"{";
	for(p=cs.begin(); p!=cs.end(); p++)
		cout<<*p<<", ";
	cout<<"}\n";
}

void printColorSets(vector<ColorSet> vcs)
{
	for(unsigned int i=0; i<vcs.size(); i++)
	{
		cout<<i<<": ";
		printColorSet(vcs[i]);
	}
}
//merge two existing colors, c1 < c2
void Merger::mergeColor(UINT c1, UINT c2)
{	
	NodeMap::iterator p;

	for(p=(*p_colorArr)[c2]->nodeMap.begin(); p!=(*p_colorArr)[c2]->nodeMap.end(); p++)
		(*p_colorArr)[c1]->nodeMap.insert(NodeMap::value_type(p->first, p->second));

	hash_map<UINT, int> p1copy((*p_colorArr)[c1]->participates);
	
	(*p_colorArr)[c1]->mergeNodeMap(c1, c2);
	
	(*p_colorArr)[c2]->nodeMap.clear();
	
	//for the neighbors of c2, 
	hash_map<UINT, int>::iterator pp2, pp1;
	ColorArray::iterator c;

	
	for(c=p_colorArr->begin(); c!=p_colorArr->end(); c++)
	{	
		UINT color=c->first;
		if(color==c1 || color==c2) continue;
		
		int pt2=0;
		for(pp2=(*p_colorArr)[c2]->participates.begin(); pp2!=(*p_colorArr)[c2]->participates.end(); pp2++)
		{
			if(pp2->first == c->first)
				pt2=pp2->second;
		}
		if(pt2==0) continue;

		//change the nodemap & the participates
		(*p_colorArr)[color]->mergeNodeMap(c1, c2);

		pp1=p1copy.find(color);
		int pt1=0;
		if(pp1!=p1copy.end())
		{
			pt1=pp1->second;
			p1copy.erase(pp1);
		}
		
		//change the participate of this merged color
		(*p_colorArr)[c1]->setParticipates(color, pt1+pt2);

	}

	UINT initColor=(*p_colorArr)[c2]->initialColor;

        initColorSets[0].erase(c2);

	ColorSet::iterator cp, cp2;

	//heap->printHeap();
	//heap->printRefMap();
	
	//remove all the distance record related to c2
	for(cp=initColorSets[0].begin(); cp!=initColorSets[0].end(); cp++)
	{
		if(*cp==c1) continue;
		ColorEdge ce;
		if(c2>*cp)
		{
			ce.colorid1=*cp;
			ce.colorid2=c2;
		}else
		{
			ce.colorid1=c2;
			ce.colorid2=*cp;
		}
		heap->removeKey(ce);
		//heap->printHeap();
		//heap->printRefMap();
	}
		p_colorArr->erase(c2);

	//first empty the heap and rebuild the heap based on the new colors
	while (heap->isEmpty()!=true)
	{
		heap->removeMin();
	}

	//for c1 and the neighbors of both colors, the distance value related to them changes
	ColorSet::iterator p1, p2;
	
	for(unsigned int i=0; i<initColorSets.size(); i++)
	{
		for(p1=initColorSets[i].begin(); p1!=initColorSets[i].end(); p1++)
		{
			p2=p1;
			p2++;
			
			if(p2!=initColorSets[i].end())
			{
				ELEM<GroupDistance, ColorEdge> elem;
				if(*p2<*p1)
				{
					elem.value.colorid1=*p2;
					elem.value.colorid2=*p1;
				}else
				{
					elem.value.colorid1=*p1;
					elem.value.colorid2=*p2;
				}

				elem.key=GroupDist((*p_colorArr)[elem.value.colorid1], (*p_colorArr)[elem.value.colorid2], elem.value.colorid1, elem.value.colorid2, p_colorArr);

				heap->insert(elem);

			}
		}
	}	

}


void Merger::printColorArr()
{
	ColorArray::iterator p;
	cout<<"colorid:";
	for(p=p_colorArr->begin(); p!=p_colorArr->end(); p++)
	{
		cout<<p->first<<"\t ";}
        cout<<"\n";
}

//mergeGroups automatically generates cutoffs. Generally, for most datasets, the small number of cutoffs (e.g. <= 4 cutoffs) is preferred 
//and suggested. Large number of cutoffs is not good at generating interesting graph summarization.
void Merger::mergeGroups(int k, Summarizer* psmr)
{
 	int delta[10000];
	int counter=0;

	if (k > (*p_colorArr).size() - 1)
	{
		cerr<<"Error: in valid k value, k should be less than "<< (*p_colorArr).size() - 1<<endl;
		exit(-1);
	}
	ColorArray::iterator p;
	ELEM<GroupDistance, ColorEdge> e;
	deltarecord=new vector< ELEM<float, ColorEdge> >();

	delta[counter]=getDelta();

	while((*p_colorArr).size() > 1 )		// iterate unitll there are only one group
	{  	 
		 ELEM<GroupDistance, ColorEdge> e=heap->removeMin();
		 counter++;
		
		//cout<<"merge c1="<<(*p_colorArr)[e.value.colorid1]->attrValues<<", c2="<<(*p_colorArr)[e.value.colorid2]->attrValues<<"\n";
	        mergeColor(e.value.colorid1, e.value.colorid2);	
		
	//special case: when # groups changes from 2 to 1 (the last merge), the value of \mu is often very large (according to the def. of \delta), 
	//no matter what the cutoff (caused by the last merge) is. Therefore, we do not store the value of \mu in the "special" merge.
	//basically, the cutoff caused by the last merge usually has a large \mu value, so, it seems to be a very "important" cutoff. However, it is not in practice. So, we ignore it 
		if ((*p_colorArr).size() != 1)  
		{				
			ELEM<float, ColorEdge> elem;
			delta[counter] = getDelta();				
			elem.key = float(delta[counter]-delta[counter-1])/float(delta[counter-1]);
			elem.value.colorid1 = e.value.colorid1;
			elem.value.colorid2 = e.value.colorid2;
			deltarecord->push_back(elem);
		}
	}
	// save all delta records into a heap
	cutoff=new MinHeap<float, ColorEdge, colorEdgehash, eqColorEdge>(deltarecord, (int)deltarecord->size(), (int)deltarecord->size());
	// find the k largest delta value ( corresponding to k splits -- k+1 groups left) in the heap and also the two colors involved in each split.
	int num_of_cuts= k;    
	assert(k <= deltarecord->size());	 // k should be no more than the size of deltarecord.

	int min_value = (*f_colorArr)[0]->attrValues;
	int max_value = (*f_colorArr)[(*f_colorArr).size()-1]->attrValues;
	vector<int>cutoffs;
	// return k cutoffs
	
	while (num_of_cuts>0)
	{
		ELEM<float, ColorEdge> cut=cutoff->removeMin();
		cutoffs.push_back((*f_colorArr)[cut.value.colorid2]->attrValues);	
		//cout<<k-num_of_cuts+1<<"cutoff(s):"<<"\t"<<"["<<min_value<<","<<(*f_colorArr)[cut.value.colorid2]->attrValues<<"]"<<endl;
		num_of_cuts--;
	}

	Sort(cutoffs);
	// output the k cutoffs
	cout<<k<<" cutoff(s): "<<"["<<min_value<<",";
	for (int i = 0; i< cutoffs.size(); i++)
	{
		cout<<cutoffs[i]-1<<"] ["<<cutoffs[i]<<",";
	}
	cout<<max_value<<"]"<<endl;
}

Merger::Merger(ColorArray* parr, ColorArray* orgarr, bool directedGraph)
{
	p_colorArr=parr;
	f_colorArr=orgarr;
	directed=directedGraph;
	ColorArray::iterator it, itloop;
        it=(*p_colorArr).begin();

	UINT initColor=it->second->initialColor;
	initColorSets.push_back(ColorSet());
	itloop=it;

	for (;itloop!=(*p_colorArr).end(); itloop++)
		initColorSets[initColor].insert(itloop->first);

	//initialize heap;
	h=new vector< ELEM<GroupDistance, ColorEdge> >();
	ColorSet::iterator p, p2;
	for(unsigned int i=0; i<initColorSets.size(); i++)
	{
		for(p=initColorSets[i].begin(); p!=initColorSets[i].end(); p++)
		{
			p2=p;
			p2++;
			if(p2!=initColorSets[i].end())
			{
				ELEM<GroupDistance, ColorEdge> elem;
				if(*p2<*p)
				{
					elem.value.colorid1=*p2;
					elem.value.colorid2=*p;
				}else
				{
					elem.value.colorid1=*p;
					elem.value.colorid2=*p2;
				}
				elem.key=GroupDist((*p_colorArr)[elem.value.colorid1], (*p_colorArr)[elem.value.colorid2], elem.value.colorid1, elem.value.colorid2, p_colorArr);
				
				h->push_back(elem);
			}
		}
	}
	heap=new MinHeap<GroupDistance, ColorEdge, colorEdgehash, eqColorEdge>(h, (int)h->size(), (int)h->size());
}


int Merger::getDelta()
{
        ColorArray::iterator p;
        hash_map<UINT, int>::iterator q;
        int distance=0;
        int numGroupEdge=0;
	if (directed==false)  // for undirected graph
	{
		for(p=(*p_colorArr).begin(); p!=(*p_colorArr).end(); p++)
        	{
                UINT color1=p->first;
                int total1=p->second->getSize();
                	for(q=p->second->participates.begin(); q!=p->second->participates.end(); q++)
                	{                                               
	                        UINT color2=q->first;
	                        if(q->second==0 || color1>color2)
	                                continue;
	                                                                                
	                        int part1=q->second;
	                        int part2=(*p_colorArr)[color2]->getParticipates(color1);
	                        assert(part2>0);
	                        int total2=(*p_colorArr)[color2]->getSize();
	                                                                                
	                        int part=part1+part2;
	                        int total=total1+total2;
	                        distance+=min(part, total-part);
	                                                                                
	                        numGroupEdge++;

                	}
        	}
	}

	else		// for directed graph
	{
        	for(p=(*p_colorArr).begin(); p!=(*p_colorArr).end(); p++)
        	{
                UINT color1=p->first;
                int total1=p->second->getSize();
                	for(q=p->second->participates.begin(); q!=p->second->participates.end(); q++)
                	{
                                                                                
	                        UINT color2=q->first;
	                        if(q->second==0)
	                                continue;
	                                                                                
	                        int part1=q->second;

	                        assert(part2>0);

	                        distance+=min(part1, total1-part1);
	                                                                                
	                        numGroupEdge++;

                	}
       		}
	}
	return distance;
}

void Merger::Sort(vector<int> &num)
{
      int i, j, flag = 1;   
      int temp;           
      int numLength = num.size( ); 
      for(i = 1; (i <= numLength) && flag; i++)
     {
          flag = 0;
          for (j=0; j < (numLength -1); j++)
         {
               if (num[j+1] < num[j])      
              { 
                    temp = num[j];            
                    num[j] = num[j+1];
                    num[j+1] = temp;
                    flag = 1;               
               }
          }
     }
     return;
}
	 

  

Merger::~Merger()
{
	delete heap;
	delete h;
}

