#include "summarizer.h"

Summarizer::~Summarizer()
{
	for(unsigned int i=0; i<colorArr.size(); i++)
	{
		delete colorArr[i];
		colorArr[i]=NULL;
	}
}

void Summarizer::groupByAttribute()
{
	UINT* nodeColorMap;
	intializeColorArray(nodeColorMap);
//	debug(33, "initialized colorarray\n");
	initializePaticipates(nodeColorMap);
//	debug(33, "initialized participates\n");
	delete [] nodeColorMap;
}
void Summarizer::intializeColorArray(UINT*& nodeColorMap)
{
	vector<int> nodes;
	vector<int> values;
	db->groupNodesByAttrnumeric(tablePrefix+"_Node", Color::gid, Color::nodeAttr, nodes, values);

	if (nodes.size() == 0 && values.size() == 0)
	{
		cerr<<"Error: wrong attribute name or graph id"<<endl;
		exit(-1);
	}

	colorArr.clear();
	
	int preValue=-1;
	
	int colorid=-1;
	ColorArray::iterator pcurrColor;

	nodeColorMap=new UINT[nodes.size()];
	for(unsigned int i=0; i<nodes.size(); i++)
	{
		if(preValue!=values[i])
		{
			colorid++;
			preValue=values[i];
			pcurrColor=(colorArr.insert(ColorArray::value_type(colorid, new Color(colorid, colorid, preValue) ) ) ).first;

		}
		pcurrColor->second->nodeMap.insert(NodeMap::value_type(nodes[i], new NodeBitArray(nodes[i])));
		nodeColorMap[nodes[i]]=colorid;
	}
}

void Summarizer::initializePaticipates(UINT* nodeColorMap)
{
	vector<int> graphs;
	vector<int> node1s;
	vector<int> node2s;
	vector<Attributes*> vattrs;
	char buff[100];
	sprintf(buff, "where graphID=%d and attrname=\'type\' and attrvalue=\'%s\'", Color::gid, Color::edgeType.c_str());
	string condition=string(buff);
	
	db->selectEdges(tablePrefix+"_Edge", condition, graphs, node1s, node2s, vattrs);

	if (node1s.size() == 0 && node2s.size() == 0 && graphs.size() == 0 )
        {
		cerr<<"Error: wrong edgetype or graph id"<<endl;
		exit(-1);
	}

	string directed;
	db->getEdgeAttr(tablePrefix+"_Edge", Color::gid, node1s[0], node2s[0], "directed", directed);  // if the graph is directed or not

	if (directed == "true")
		directedGraph = true;
	else
		directedGraph = false;

	UINT colorid1, colorid2;
	UINT v1, v2;
	
	for(int i=0; i<graphs.size(); i++)
	{
		v1=node1s[i];
		v2=node2s[i];

		colorid1=nodeColorMap[v1];
		colorid2=nodeColorMap[v2];
		//modify the color array
		colorArr[colorid1]->nodeMap[v1]->setBitOn(colorid2);
		if (!directedGraph)
		colorArr[colorid2]->nodeMap[v2]->setBitOn(colorid1);
	}
	
	//update the frequency counts of each colorArr
	ColorArray::iterator it;
	for(it=colorArr.begin(); it!=colorArr.end(); it++)
	{
		it->second->updateParticipates();
	}
	//clean the attribute pointers
	for(unsigned int i=0; i<vattrs.size(); i++)
	{
		delete vattrs[i];
	}
}
