#include "loadgraph/load.h"
#include "graph/hashgraph.h"
#include "config.h"
#include "db/db.h"
#include "db/pgsql_db.h"
#include "util/mytimer.h"
#include "debug/debug.h"
#include "io/graphio.h"

int main(int argc, char** argv) 
{
	//cout<<"distrec size: "<<sizeof(DistRec)<<endl;
	HashGraph* G;
	OrthologInfoList orthinfolist;

	if (argc != 7) {
	cout << "Usage: bulkload <configuration file> <a list of graph files> <orthology file> <table Prefix>"
			"<index choice:0(no index), 1(saga index), 2(tale index), 3(both index)> <directed graph:1. undirected graph:0>\n";
	exit(-1);
	}
	
	cout<<"-- Load Graphs Listed in "<<argv[2]<<endl;

	FILE* flist=fopen(argv[2], "r");
	
	if(!flist)
	{
	   cout << "Unable to open file |" << argv[2] << "|\n";
	   exit(-1);
	}
	
	Config cf;
	cf.readFile(argv[1]);
	//cf.printConfig();
	string orth_file_name=string(argv[3]);
	string tablePrefix=string(argv[4]);
	FILE* orthf=fopen(orth_file_name.c_str(), "r");
	
	if(!orthf)
	{
		cout<<"Unable to open file "<<orth_file_name<<endl;
		exit(-1);
	}
	
	int choice=atoi(argv[5]);
	
	bool direct=false;	
	if(atoi(argv[6])==1)
	direct=true;

	bool forsaga=false, fortale=false;
	
	forsaga=(choice%2);
	fortale=(choice/2);

	string path=cf.tempdir;
	if(path[path.size()-1]!='/')
		path=path+"/";
	
	string name=path+"node.bin";
	remove(name.c_str());
	FILE* nfile=fopen(name.c_str(),"w");
	
	name=path+"edge.bin";
    remove(name.c_str());
    FILE* efile=fopen(name.c_str(),"w");
    
    name=path+"orth.bin";
    remove(name.c_str());
    FILE* ofile=fopen(name.c_str(),"w");
    
    name=path+"dist.bin";
    remove(name.c_str());
    FILE* dfile=fopen(name.c_str(),"w");
    
    name=path+"frag.bin";
    remove(name.c_str());
    FILE* ffile=fopen(name.c_str(),"w");
    
    name=path+"neig.bin";
    remove(name.c_str());
    FILE* nbfile=fopen(name.c_str(),"w");
    
    if( !nfile || !efile || !ofile || !dfile || !ffile || !nbfile)
	{
	   cout << "Do not have enouph space to create temporary files\n";
	   exit(-1);
	}

	
	PGDB* db=new PGDB((char*)cf.dbname.c_str(), (char*)cf.dbuser.c_str(), (char*)cf.dbpwd.c_str());
	WriteToFile* wtf=new WriteToFile(nfile, efile, ofile, dfile, ffile, nbfile);
	//make headers
	char buff[19];
    memcpy(buff, "PGCOPY\n\377\r\n\0\0\0\0\0\0\0\0\0", sizeof(buff));
                                                                                
    fwrite(buff, sizeof(buff), 1, nfile);
    fwrite(buff, sizeof(buff), 1, efile);
	fwrite(buff, sizeof(buff), 1, ofile);
    fwrite(buff, sizeof(buff), 1, dfile);
    fwrite(buff, sizeof(buff), 1, ffile);
    fwrite(buff, sizeof(buff), 1, nbfile);
	
	//debug(16, "before loading ecmap\n");
	Ortholog orth;
	orth.buildOrthologMap(orthf);
	
	MicroTimer loadTimer;

	LoadGraph loadgrf(cf.frag_size, cf.dmax, db, wtf, tablePrefix, direct);
	
    loadgrf.createTables();	
	//cout<<"Created Tables\n";
	int gid=db->getMaxGraphID(tablePrefix+"_graph");
    
	debug(33, "gid = "<<gid<<endl);
	
    gid++;
	loadTimer.reset();
	char fname[250];
	
	while( fscanf(flist, "%s\n", fname)==1)
	{
		FILE* fin=fopen(fname, "r");
		if(!fin)
		{
			cout<<"Unable to open file "<<fname<<endl;
			exit(-1);
		}
		string str=string(fname);
		int start=str.find_last_of("/");
		if(start<0)
			start=0;
		else
			start=start+1;
		int end=str.find_last_of(".");
		if(end<0)
			end=str.size();
		
		string gname=str.substr(start, end-start);
			
	//	while((G=readGraphSAGAFormat(fin, orth, orthinfolist))!=NULL)
		while((G=readGraphGDFFormat(fin, gname, orth, orthinfolist, direct))!=NULL)
		{
			loadgrf.setGraph(gid, G, &orthinfolist, &orth);
			debug(33, "set graph succeeded!\n");
			loadgrf.load(forsaga, fortale);
			debug(33, "load graph succeeded\n");		
			delete G;
			gid++;
		
		}
		fclose(fin);
	}
	
	fclose(flist);
	fclose(orthf);
	fclose(nfile);
	fclose(efile);
	fclose(ofile);
	fclose(dfile);
	fclose(ffile);
	fclose(nbfile);

	loadgrf.copyData(path);
 //   cout<<"Copied Data\n";
	
	loadgrf.createIndices();
	
	if(forsaga)
		cout<<"   Build Index for SAGA Query\n";
	if(fortale)
		cout<<"   Build Index for TALE Query\n";
	
//    cout<<"Created Indices\n";
	cout<<"-- Loading Time: "<<loadTimer.stop_seconds()<<endl;

	delete db; db=NULL;
	delete wtf; wtf=NULL;
	return 0;
}


