#include "autocut/summarizer.h"
#include "autocut/merger.h"
#include "visual/visual.h"
#include "graph/hashgraph.h"
#include "config.h"
#include "db/db.h"
#include "autocut/color.h"
#include "db/pgsql_db.h"
#include "util/graphutil.h"
#include "io/summaryio.h"
#include "util/mytimer.h"

int main(int argc, char** argv) 
{
	
	if (argc !=7 ) 
	{
        cerr << "Usage: canal [configuration file][table Prefix] [graph id] [attrName] [edgeType] [k: number of cutoff]\n";
        exit(-1);
   	}
	
	Config cf;
    	cf.readFile(argv[1]);

	PGDB* db=new PGDB((char*)cf.dbname.c_str(), (char*)cf.dbuser.c_str(), (char*)cf.dbpwd.c_str());
	string tablePrefix=string(argv[2]);
	int g=atoi(argv[3]);
	string attrname=string(argv[4]);
	string edgetype=string(argv[5]);

	int k=atoi(argv[6]);
	if(k<=1)
	{
		cerr<<"Error: in valid k value (k>1): "<<k<<endl;
		exit(-1);
	}
	//////////////////////////////////////
	Summarizer smr(db);

	//set the graph attributes and load the graph into memory
	smr.setGraph(tablePrefix, g, attrname, edgetype);
	debug(33, "set graph succeed\n");

	MicroTimer timer;
        timer.reset();
	smr.origin_colorArr = smr.colorArr;
	Merger mgr(&(smr.colorArr), &(smr.origin_colorArr),smr.directedGraph);
	
	//mergeGroups automatically generates cutoffs. Generally, for most datasets, the small number of cutoffs (e.g. <= 4 cutoffs) is preferred 
	//and suggested. Large number of cutoffs is not good at generating interesting graph summarization.
        mgr.mergeGroups(k,&smr);
		
	debug(33, "canal succeed\n");
	cout<<"-- Time Spent Totally (s): "<<timer.stop_seconds()<<endl;
	delete db;
	return 0;
}
