#include "graph/hashgraph.h"
#include "config.h"
#include "db/db.h"
#include "db/pgsql_db.h"
#include "util/mytimer.h"
#include "debug/debug.h"
#include "io/graphio.h"

int main(int argc, char** argv) 
{
	//cout<<"distrec size: "<<sizeof(DistRec)<<endl;
	HashGraph* G;
	OrthologInfoList orthinfolist;

	if (argc != 8) {
	cout << "Usage: creategraph <configuration file> <orthology file> <table Prefix> <graph id> <node condidtion> <edge condition> <output file>\n";
	exit(-1);
	}
	
	int gid=atoi(argv[4]);
	string nodeCondition=string(argv[5]);
	string edgeCondition=string(argv[6]);
	
	Config cf;
	cf.readFile(argv[1]);
	//cf.printConfig();
	FILE* orthf=NULL;
	string orth_file_name=string(argv[2]);
	string tablePrefix=string(argv[3]);
	orthf=fopen(orth_file_name.c_str(), "r");
	if(!orthf)
	{
	   cout << "Error: Unable to open file |" << orth_file_name << "|\n";
	   exit(-1);
	}
	
	PGDB* db=new PGDB((char*)cf.dbname.c_str(), (char*)cf.dbuser.c_str(), (char*)cf.dbpwd.c_str());
	
	//debug(16, "before loading ecmap\n");
	Ortholog orth;
	orth.buildOrthologMap(orthf);
	
	MicroTimer loadTimer;
	
	cout<<"-- Create Graph File: "<<argv[7]<<endl;
	if((G=createGraphFromDB(db, tablePrefix, gid, nodeCondition, edgeCondition, orthinfolist))!=NULL)
	{
		cout<<"   # Nodes: "<<G->n()<<endl;
		cout<<"   # Edges: "<<G->e()<<endl;
		ofstream of(argv[7]);
		if(!of.is_open())
		{
			cout<<"Error: Can not open file "<<argv[7]<<endl;
			exit(-1);
		}
		
		writeGraphGDFFormat(of, G, orth, orthinfolist);
		of.close();
		delete G;
		gid++;
	
	}
	
	fclose(orthf);
	
	cout<<"-- Time Spent Totally (s): "<<loadTimer.stop_seconds()<<endl;

	delete db; db=NULL;
	return 0;
}


