/******************************************************************************
 * The implementation of class PGDB using PostgreSQL
 * Author: Yuanyuan Tian (ytian@umich.edu)
 * ***************************************************************************/
#include "pgsql_db.h"
#include <assert.h>
#include <cmath>
#include "../debug/debug.h"

void getIntVector(char* ptr, vector<int>& vec)
{
	/*----------
	 * A standard varlena array has the following internal structure:
	 *	  <size>		- total number of bytes (also, TOAST info flags): 20+N*8
	 *	  <ndim>		- number of dimensions of the array: 1
	 *	  <flags>		- bit mask of flags: 0
	 *	  <elemtype>	- element type OID: \0\0\00027 = 23
	 *	  <dim>			- size of each array axis (C array of int): N
	 *	  <dim_lower>	- lower boundary of each dimension (C array of int):1
	 *	  <actual data> - whatever is the stored data: [4,v]
	 * The actual data starts on a MAXALIGN boundary.  Individual items in the
	 * array are aligned as specified by the array element type.
	 */
	
	//<size><ndim><flags><elemtype><dim><dim_lower>
	uint32_t* p=(uint32_t*)ptr;
	int size=ntohl(p[3]); //the number of int
	debug(1, "p[0]="<<ntohl(p[0])<<endl);
	debug(1, "p[1]="<<ntohl(p[1])<<endl);
	debug(1, "p[2]="<<ntohl(p[2])<<endl);
	debug(1, "p[3]="<<ntohl(p[3])<<endl);
	debug(1, "p[4]="<<ntohl(p[4])<<endl);
	debug(1, "p[5]="<<ntohl(p[5])<<endl);
	debug(1, "in getIntVector: size = "<<size<<endl);
	int j=5;
	for(int i=0; i<size; i++)
	{
		int length=ntohl(p[j]);
		assert(length==sizeof(uint32_t));
		int value=ntohl(p[j+1]);
		vec.push_back(value);
		j=j+2;
	}
	/*
	 * //<size><ndim><flags><elemtype><dim><dim_lower>
	uint32_t head[6];
	head[0]=htonl(uint32_t ( 20 + (rec.nidSeq.size())*8 ) );
	head[1]=htonl((uint32_t)1);
	head[2]=htonl((uint32_t)0);
	head[3]=htonl((uint32_t)23);
	head[4]=htonl((uint32_t)rec.nidSeq.size());
	head[5]=htonl((uint32_t)1);
	
	memcpy(p, head, sizeof(head));
	p+=sizeof(head);
	
	//<actual data>
	length=htonl(sizeof(uint32_t));
	for(unsigned int i=0; i<rec.nidSeq.size(); i++)
	{
		memcpy(p, &(length), sizeof(length));
		p+=sizeof(length);
		memcpy(p, &(rec.nidSeq[i]), sizeof(rec.nidSeq[i]));
		p+=sizeof(rec.nidSeq[i]);
	}
	 */
}

void WriteToFile::writeNodeRec(NodeRec rec)
{
	char buff[500];
	
	uint16_t nfield=htons(5);
	uint32_t length;
	
	rec.graphID=htonl(rec.graphID);
	rec.nodeID=htonl(rec.nodeID);
	
	char* p=buff;
	memcpy(p, &(nfield), sizeof(nfield));
	p+=sizeof(nfield);
	
	length=htonl(sizeof(rec.graphID));
	memcpy(p, &(length), sizeof(length));
	p+=sizeof(length);
	memcpy(p, &(rec.graphID), sizeof(rec.graphID));
	p+=sizeof(rec.graphID);
	
	length=htonl(sizeof(rec.nodeID));
	memcpy(p, &(length), sizeof(length));
	p+=sizeof(length);
	memcpy(p, &(rec.nodeID), sizeof(rec.nodeID));
	p+=sizeof(rec.nodeID);
	
	length=htonl(rec.attrName.size());
	memcpy(p, &(length), sizeof(length));
	p+=sizeof(length);
	memcpy(p, rec.attrName.c_str(), rec.attrName.size());
	p+=rec.attrName.size();
	
	length=htonl(sizeof(rec.attrType));
	memcpy(p, &(length), sizeof(length));
	p+=sizeof(length);
	memcpy(p, &rec.attrType, sizeof(rec.attrType));
	p+=sizeof(rec.attrType);

	length=htonl(rec.attrValue.size());
	memcpy(p, &(length), sizeof(length));
	p+=sizeof(length);
	memcpy(p, rec.attrValue.c_str(), rec.attrValue.size());
	p+=rec.attrValue.size();
	
	int size = p - buff;
	fwrite(buff, size, 1, nodeFile);
}


void WriteToFile::writeEdgeRec(EdgeRec rec)
{
	char buff[500];
	
	uint16_t nfield=htons(6);
	uint32_t length;
	
	rec.graphID=htonl(rec.graphID);
	rec.node1ID=htonl(rec.node1ID);
	rec.node2ID=htonl(rec.node2ID);

	char* p=buff;
	memcpy(p, &(nfield), sizeof(nfield));
	p+=sizeof(nfield);
	
	length=htonl(sizeof(rec.graphID));
	memcpy(p, &(length), sizeof(length));
	p+=sizeof(length);
	memcpy(p, &(rec.graphID), sizeof(rec.graphID));
	p+=sizeof(rec.graphID);
	
	length=htonl(sizeof(rec.node1ID));
	memcpy(p, &(length), sizeof(length));
	p+=sizeof(length);
	memcpy(p, &(rec.node1ID), sizeof(rec.node1ID));
	p+=sizeof(rec.node1ID);
	
	length=htonl(sizeof(rec.node2ID));
	memcpy(p, &(length), sizeof(length));
	p+=sizeof(length);
	memcpy(p, &(rec.node2ID), sizeof(rec.node2ID));
	p+=sizeof(rec.node2ID);
	
	length=htonl(rec.attrName.size());
	memcpy(p, &(length), sizeof(length));
	p+=sizeof(length);
	memcpy(p, rec.attrName.c_str(), rec.attrName.size());
	p+=rec.attrName.size();
	
	length=htonl(sizeof(rec.attrType));
	memcpy(p, &(length), sizeof(length));
	p+=sizeof(length);
	memcpy(p, &rec.attrType, sizeof(rec.attrType));
	p+=sizeof(rec.attrType);

	length=htonl(rec.attrValue.size());
	memcpy(p, &(length), sizeof(length));
	p+=sizeof(length);
	memcpy(p, rec.attrValue.c_str(), rec.attrValue.size());
	p+=rec.attrValue.size();
	
	int size = p - buff;
	fwrite(buff, size, 1, edgeFile);
}


void WriteToFile::writeOrthologRec(OrthologRec rec)
{
	char buff[500];
	
	uint16_t nfield=htons(3);
	uint32_t length;
	
	rec.orthologID=htonl(rec.orthologID);
	
	char* p=buff;
	memcpy(p, &(nfield), sizeof(nfield));
	p+=sizeof(nfield);
	
	length=htonl(sizeof(rec.orthologID));
	memcpy(p, &(length), sizeof(length));
	p+=sizeof(length);
	memcpy(p, &(rec.orthologID), sizeof(rec.orthologID));
	p+=sizeof(rec.orthologID);
	
	length=htonl(rec.orthologName.size());
	memcpy(p, &(length), sizeof(length));
	p+=sizeof(length);
	memcpy(p, rec.orthologName.c_str(), rec.orthologName.size());
	p+=rec.orthologName.size();
	
	length=htonl(rec.source.size());
	memcpy(p, &(length), sizeof(length));
	p+=sizeof(length);
	memcpy(p, rec.source.c_str(), rec.source.size());
	p+=rec.source.size();
	
	int size = p - buff;
	fwrite(buff, size, 1, orthFile);
}


void WriteToFile::writeDistanceRec(DistanceRec rec)
{
	debug(1, "in write DistanceRec\n");
	debug(1, "distance="<<rec.distance<<endl);
	char buff[500];
	
	uint16_t nfield=htons(4);
	uint32_t length;

	rec.graphID=htonl(rec.graphID);
	rec.node1ID=htonl(rec.node1ID);
	rec.node2ID=htonl(rec.node2ID);
	rec.distance=htonl(rec.distance);
	
	char* p=buff;
	memcpy(p, &(nfield), sizeof(nfield));
	p+=sizeof(nfield);
	
	length=htonl(sizeof(rec.graphID));
	memcpy(p, &(length), sizeof(length));
	p+=sizeof(length);
	memcpy(p, &(rec.graphID), sizeof(rec.graphID));
	p+=sizeof(rec.graphID);
	
	length=htonl(sizeof(rec.node1ID));
	memcpy(p, &(length), sizeof(length));
	p+=sizeof(length);
	memcpy(p, &(rec.node1ID), sizeof(rec.node1ID));
	p+=sizeof(rec.node1ID);
	
	length=htonl(sizeof(rec.node2ID));
	memcpy(p, &(length), sizeof(length));
	p+=sizeof(length);
	memcpy(p, &(rec.node2ID), sizeof(rec.node2ID));
	p+=sizeof(rec.node2ID);
	
	length=htonl(sizeof(rec.distance));
	memcpy(p, &(length), sizeof(length));
	p+=sizeof(length);
	memcpy(p, &(rec.distance), sizeof(rec.distance));
	p+=sizeof(rec.distance);
	
	int size = p - buff;
	fwrite(buff, size, 1, distFile);
}

void WriteToFile::writeNeighborhoodRec(NeighborhoodRec rec)
{
	char buff[500];
	
	uint16_t nfield=htons(6);
	uint32_t length;
	
	rec.orthologID=htonl(rec.orthologID);
	rec.degree=htonl(rec.degree);
	rec.nbConnection=htonl(rec.nbConnection);

	for(int i=0; i<96/16; i++)
			rec.nbArray[i]=htons(rec.nbArray[i]);

	rec.graphID=htonl(rec.graphID);
	rec.nodeID=htonl(rec.nodeID);
	
	char* p=buff;
	memcpy(p, &(nfield), sizeof(nfield));
	p+=sizeof(nfield);
	
	length=htonl(sizeof(rec.orthologID));
	memcpy(p, &(length), sizeof(length));
	p+=sizeof(length);
	memcpy(p, &(rec.orthologID), sizeof(rec.orthologID));
	p+=sizeof(rec.orthologID);
	
	length=htonl(sizeof(rec.degree));
	memcpy(p, &(length), sizeof(length));
	p+=sizeof(length);
	memcpy(p, &(rec.degree), sizeof(rec.degree));
	p+=sizeof(rec.degree);
	
	length=htonl(sizeof(rec.nbConnection));
	memcpy(p, &(length), sizeof(length));
	p+=sizeof(length);
	memcpy(p, &(rec.nbConnection), sizeof(rec.nbConnection));
	p+=sizeof(rec.nbConnection);
	
	length=htonl(sizeof(rec.nbArray)+sizeof(uint32_t));
	memcpy(p, &(length), sizeof(length));
	p+=sizeof(length);
	uint32_t numbits=htonl(96);
	memcpy(p, &(numbits), sizeof(numbits));
	p+=sizeof(numbits);
	memcpy(p, rec.nbArray, sizeof(rec.nbArray));
	p+=sizeof(rec.nbArray);
	
	length=htonl(sizeof(rec.graphID));
	memcpy(p, &(length), sizeof(length));
	p+=sizeof(length);
	memcpy(p, &(rec.graphID), sizeof(rec.graphID));
	p+=sizeof(rec.graphID);
	
	length=htonl(sizeof(rec.nodeID));
	memcpy(p, &(length), sizeof(length));
	p+=sizeof(length);
	memcpy(p, &(rec.nodeID), sizeof(rec.nodeID));
	p+=sizeof(rec.nodeID);
	
	int size=p - buff;

	fwrite(buff, size, 1, neighborFile);
}

void WriteToFile::writeFragmentRec(FragmentRec rec)
{
	char buff[500];
	
	uint16_t nfield=htons(5);
	uint32_t length;
	
	rec.orthSeq=htonll(rec.orthSeq);
	rec.distSum=htonl(rec.distSum);
	rec.graphID=htonl(rec.graphID);

	for(unsigned int i=0; i<rec.nidSeq.size(); i++)
		rec.nidSeq[i]=htonl(rec.nidSeq[i]);
	
	for(unsigned int i=0; i<rec.distMatrix.size(); i++)
		rec.distMatrix[i]=htonl(rec.distMatrix[i]);
	
	char* p=buff;
	memcpy(p, &(nfield), sizeof(nfield));
	p+=sizeof(nfield);
	
	length=htonl(sizeof(rec.orthSeq));
	memcpy(p, &(length), sizeof(length));
	p+=sizeof(length);
	memcpy(p, &(rec.orthSeq), sizeof(rec.orthSeq));
	p+=sizeof(rec.orthSeq);
	
	length=htonl(sizeof(rec.distSum));
	memcpy(p, &(length), sizeof(length));
	p+=sizeof(length);
	memcpy(p, &(rec.distSum), sizeof(rec.distSum));
	p+=sizeof(rec.distSum);
	
	length=htonl(sizeof(rec.graphID));
	memcpy(p, &(length), sizeof(length));
	p+=sizeof(length);
	memcpy(p, &(rec.graphID), sizeof(rec.graphID));
	p+=sizeof(rec.graphID);
	
	/*----------
	 * A standard varlena array has the following internal structure:
	 *	  <size>		- total number of bytes (also, TOAST info flags): 20+N*8
	 *	  <ndim>		- number of dimensions of the array: 1
	 *	  <flags>		- bit mask of flags: 0
	 *	  <elemtype>	- element type OID: \0\0\00027 = 23
	 *	  <dim>			- size of each array axis (C array of int): N
	 *	  <dim_lower>	- lower boundary of each dimension (C array of int):1
	 *	  <actual data> - whatever is the stored data: [4,v]
	 * The actual data starts on a MAXALIGN boundary.  Individual items in the
	 * array are aligned as specified by the array element type.
	 */
	
	//<size><ndim><flags><elemtype><dim><dim_lower>
	uint32_t head[6];
	head[0]=htonl(uint32_t ( 20 + (rec.nidSeq.size())*8 ) );
	head[1]=htonl((uint32_t)1);
	head[2]=htonl((uint32_t)0);
	head[3]=htonl((uint32_t)23);
	head[4]=htonl((uint32_t)rec.nidSeq.size());
	head[5]=htonl((uint32_t)1);
	
	memcpy(p, head, sizeof(head));
	p+=sizeof(head);
	
	//<actual data>
	length=htonl(sizeof(uint32_t));
	for(unsigned int i=0; i<rec.nidSeq.size(); i++)
	{
		memcpy(p, &(length), sizeof(length));
		p+=sizeof(length);
		memcpy(p, &(rec.nidSeq[i]), sizeof(rec.nidSeq[i]));
		p+=sizeof(rec.nidSeq[i]);
	}
	
	head[0]=htonl(uint32_t ( 20+rec.distMatrix.size()*8 ));
	head[4]=htonl((uint32_t)rec.distMatrix.size());
	
	memcpy(p, head, sizeof(head));
	p+=sizeof(head);
	
	for(unsigned int i=0; i<rec.distMatrix.size(); i++)
	{
		memcpy(p, &(length), sizeof(length));
		p+=sizeof(length);
		memcpy(p, &(rec.distMatrix[i]), sizeof(rec.distMatrix[i]));
		p+=sizeof(rec.distMatrix[i]);
	}
	int size=p - buff;

	fwrite(buff, size, 1, fragmentFile);
}

//create the graph table 
//schema: graph(graphID, attrName, attrType, attrValue)
//input: the table name
void PGDB::createGraphTable(string tablename)
{
	char qrystr[100];
	sprintf(qrystr, "create table %s(graphID int4, attrName varchar, "
			"attrType char(1), attrValue varchar)", tablename.c_str());
	nonQueryCmd(qrystr);
}

//insert one graph into the graph table
void PGDB::insertIntoGraphTable(
		string tablename, 
		int graphID, 
		const Attributes& attrs //the attrbutes of the graph
)
{
	char qrystr[500];
	Attributes::const_iterator iter;
	for(iter=attrs.begin(); iter!=attrs.end(); iter++)
	{
		sprintf(qrystr, "insert into %s (graphID, attrName, attrType, attrValue) "
				"values(%d, \'%s\', \'%c\', \'%s\')", 
				tablename.c_str(), graphID, iter->first.c_str(), 
                        	iter->second.type, iter->second.value.c_str());
                        
		nonQueryCmd(qrystr);
	}
}

//create an index for getting graph attribute(s)
//graph_index(graphID, attrName, attrType, attrValue) on graph
void PGDB::createGraphIndex(string tablename)
{
	char qrystr[500];
	sprintf(qrystr, "create index %s_index on %s(graphID, attrName, "
			"attrType, attrValue)", tablename.c_str(), tablename.c_str());
	nonQueryCmd(qrystr);
}

//access one attribute of a graph
void PGDB::getGraphAttr(
		string tablename, 
		int graphID, 
		string attrName,
		Attributes& attrs, //return value (passed by reference)
		string preparedplan //the name of a prepared plan
							//NULL if you don't want to use
							//prepared plan
)
{
	char qrystr[500];
	if(preparedplan.empty())
	{
		sprintf(qrystr, "select attrType, attrValue from %s where graphID=%d and "
				"attrName=\'%s\'", tablename.c_str(), graphID, attrName.c_str());
	}else
	{
		sprintf(qrystr, "execute %s (%d, \'%s\')", preparedplan.c_str(), 
				graphID, attrName.c_str());
	}
	                                                                        
	PGresult* res=queryCmd(qrystr);
	
	assert(PQnfields(res)==2);

	for(int i=0; i<PQntuples(res); i++)
	{
		AttributeValue attr;
		char* ptr=PQgetvalue(res, i, 0);
		attr.type=ptr[0];
		ptr=PQgetvalue(res, i, 1);
		attr.value=string(ptr);
		attrs.insert(Attributes::value_type(attrName, attr));
	}
	
	PQclear(res);
}

//access all the attributes of a graph
void PGDB::getGraphAttr(
		string tablename, // input
		int graphID, //input
		Attributes& attrs, //return value (passed by reference)
		string preparedplan //the name of a prepared plan
							//NULL if you don't want to use
							//prepared plan
)
{
	char qrystr[500];
	if(preparedplan.empty())
	{
		sprintf(qrystr, "select attrName, attrType, attrValue from %s where graphID=%d", 
				tablename.c_str(), graphID);
	}else
	{
		sprintf(qrystr, "execute %s (%d)", preparedplan.c_str(), graphID);
	}
	                                                                        
	PGresult* res=queryCmd(qrystr);
	
	assert(PQnfields(res)==3);

	for(int i=0; i<PQntuples(res); i++)
	{
		AttributeValue attr;
		char* ptr=PQgetvalue(res, i, 0);
		string attrName=string(ptr);
		ptr=PQgetvalue(res, i, 1);
		attr.type=ptr[0];
		ptr=PQgetvalue(res, i, 2);
		attr.value=string(ptr);
		attrs.insert(Attributes::value_type(attrName, attr));
	}
	
	PQclear(res);
}

//get the maximun graphID in the graph table
int PGDB::getMaxGraphID(string tablename)
{                                                                        
    char qrystr[500];
    sprintf(qrystr, "select count(graphID) from %s", tablename.c_str());
    PGresult* res=queryCmd(qrystr);
    assert(PQntuples(res)==1);
    assert(PQnfields(res)==1);
    char* iptr=PQgetvalue(res, 0, 0);                                                                        
    int ival=(int) ntohl(*((uint32_t*)iptr));
    if(ival==0)
    {
            PQclear(res);
            return 0;
    }
    
    PQclear(res);
    
    sprintf(qrystr, "select max(graphID) from %s", tablename.c_str());
    
    res=queryCmd(qrystr);
    
    assert(PQntuples(res)==1);
    assert(PQnfields(res)==1);
                                                                            
    iptr=PQgetvalue(res, 0, 0);                                                                        
    ival=(int) ntohl(*((uint32_t*)iptr));
    PQclear(res);                                     
    return ival;
}

//select graphs based on the given condition
void PGDB::selectGraphs(
		string tablename, //input
		string condition, //the input sql condition on the graph table
						  //e.g. "where attrName='name'"
		vector<int>& graphs //return value
)
{
	char qrystr[500];
	sprintf(qrystr, "select distinct graphID from %s %s", 
				tablename.c_str(), condition.c_str());                                                                        
	PGresult* res=queryCmd(qrystr);
	for(int i=0; i<PQntuples(res); i++)
	{
		char* iptr=PQgetvalue(res, i, 0);
		int ival=(int) ntohl(*((uint32_t*)iptr));
		graphs.push_back(ival);
	}
	
	PQclear(res);
}


//create the node table
//schema: node(graphID, nodeID, attrName, attrType, attrValue)
//input: the table name
void PGDB::createNodeTable(string tablename)
{
	char qrystr[500];
	sprintf(qrystr, "create table %s(graphID int4, nodeID int4, "
			"attrName varchar, attrType char(1),attrValue varchar)", 
			tablename.c_str());
	nonQueryCmd(qrystr);	
}

//insert one node into the node table
//node_index(graphID, nodeID, attrName, attrType, attrValue)
void PGDB::insertIntoNodeTable(
		string tablename, 
		int graphID, 
		int nodeID, 
		const Attributes& attrs //the attributes of the node
)
{
	char qrystr[500];
	Attributes::const_iterator iter;
	for(iter=attrs.begin(); iter!=attrs.end(); iter++)
	{
		sprintf(qrystr, "insert into %s (graphID, nodeID, attrName, attrType,"
				" attrValue) values(%d, %d, \'%s\', \'%c\', \'%s\')", 
				tablename.c_str(), graphID, nodeID, iter->first.c_str(), 
				iter->second.type, iter->second.value.c_str());
		nonQueryCmd(qrystr);
	}
}

//create an index for getting node attribute(s)
void PGDB::createNodeIndex(string tablename)
{
	char qrystr[500];
	sprintf(qrystr, "create index %s_index on %s(graphID, nodeID, attrName, "
			"attrType, attrValue)", tablename.c_str(), tablename.c_str());
	nonQueryCmd(qrystr);
}

//access one attribute of a node
void PGDB::getNodeAttr(
		string tablename, 
		int graphID, 
		int nodeID, 
		string attrName,
		Attributes& attrs, //return value (passed by reference)
		string preparedplan //the name of a prepared plan
							//NULL if you don't want to use
							//prepared plan
)
{
	char qrystr[500];
	if(preparedplan.empty())
	{
		sprintf(qrystr, "select attrType, attrValue from %s where graphID=%d and "
				"nodeID=%d and attrName=\'%s\'", tablename.c_str(), graphID, 
				nodeID, attrName.c_str());
	}else
	{
		sprintf(qrystr, "execute %s (%d, %d, \'%s\')", preparedplan.c_str(), 
				graphID, nodeID, attrName.c_str());
	}
	                                                                        
	PGresult* res=queryCmd(qrystr);
	
	assert(PQnfields(res)==2);

	for(int i=0; i<PQntuples(res); i++)
	{
		AttributeValue attr;
		char* ptr=PQgetvalue(res, i, 0);
		attr.type=ptr[0];
		ptr=PQgetvalue(res, i, 1);
		attr.value=string(ptr);
		attrs.insert(Attributes::value_type(attrName, attr));
	}
	
	PQclear(res);
}

//access all the attributes of a node
void PGDB::getNodeAttr(
		string tablename, //input
		int graphID, //input
		int nodeID, //input
		Attributes& attrs, //return value (passed by reference)
		string preparedplan //the name of a prepared plan
							//NULL if you don't want to use
							//prepared plan
)
{
	char qrystr[500];
	if(preparedplan.empty())
	{
		sprintf(qrystr, "select attrName, attrType, attrValue from %s where "
				"graphID=%d and nodeID=%d", tablename.c_str(), graphID, nodeID);
	}else
	{
		sprintf(qrystr, "execute %s (%d, %d)", 
				preparedplan.c_str(), graphID, nodeID);
	}
	                                                                        
	PGresult* res=queryCmd(qrystr);
	
	assert(PQnfields(res)==3);

	for(int i=0; i<PQntuples(res); i++)
	{
		AttributeValue attr;
		char* ptr=PQgetvalue(res, i, 0);
		string attrName=string(ptr);
		ptr=PQgetvalue(res, i, 1);
		attr.type=ptr[0];
		ptr=PQgetvalue(res, i, 2);
		attr.value=string(ptr);
		attrs.insert(Attributes::value_type(attrName, attr));
	}
	
	PQclear(res);
}

//select nodes
void PGDB::selectNodes(
		string tablename, //input
		string condition, //the input sql condition on the node table
		vector<int>& graphs,//return value
		vector<int>& nodes, //return value
		vector<Attributes*>& vattrs	//return value
		)
{
	char qrystr[500];
	sprintf(qrystr, "select graphID, nodeID, attrname, attrtype, attrvalue "
			"from %s %s order by graphID, nodeID", tablename.c_str(), condition.c_str());                                                                        
	PGresult* res=queryCmd(qrystr);
	storeNodes(res, graphs, nodes, vattrs);
	PQclear(res);
}

//store the edges in vectors from the database results
void PGDB::storeNodes(
		PGresult* res, //database resource
		vector<int>& graphs,//return value
		vector<int>& nodes, //return value
		vector<Attributes*>& vattrs	//return value
		)
{
	int pregid=0, prenid=0;
	Attributes* pattrs=NULL;
	for(int i=0; i<PQntuples(res); i++)
	{
		char* ptr=PQgetvalue(res, i, 0);
		int gid=(int) ntohl(*((uint32_t*)ptr));
		ptr=PQgetvalue(res, i, 1);
		int nid=(int) ntohl(*((uint32_t*)ptr));
		ptr=PQgetvalue(res, i, 2);
		string attrName=string(ptr);
		AttributeValue attr;
		ptr=PQgetvalue(res, i, 3);
		attr.type=ptr[0];
		ptr=PQgetvalue(res, i, 4);
		attr.value=string(ptr);
		debug(2, "graph: "<<gid<<" ("<<nid<<") attribute "<<attrName<<endl);
		if(gid!=pregid || prenid!=nid)
		{
			debug(2, "insert\n");
			graphs.push_back(gid);
			nodes.push_back(nid);
			pattrs=new Attributes;
			vattrs.push_back(pattrs);
			pregid=gid;
			prenid=nid;
		}
		pattrs->insert(Attributes::value_type(attrName, attr));
	}
}

//group nodes by attribute values, for categorical values
void PGDB::groupNodesByAttr(
		string tablename, //input
		int graphID, //input
		string attrName, //input
		vector<int>& nodes, //return value
		vector<string>& attrValues //return value
		)
{
	char qrystr[500];
	sprintf(qrystr, "select nodeID, attrValue from %s where graphID=%d and attrName=\'%s\' order by attrValue;", 
					tablename.c_str(), graphID, attrName.c_str()); 
	PGresult* res=queryCmd(qrystr);

	
	for(int i=0; i<PQntuples(res); i++)
	{
		char* iptr, *sptr;
		iptr=PQgetvalue(res, i, 0);
		int v= (int) ntohl(*((uint32_t*)iptr));
		sptr=PQgetvalue(res, i, 1);
		nodes.push_back(v);
		attrValues.push_back(string(sptr));
	}
	PQclear(res);
}

//group nodes by attribute values, for numerical values
void PGDB::groupNodesByAttrnumeric(
		string tablename, //input
		int graphID, //input
		string attrName, //input
		vector<int>& nodes, //return value
		vector<int>& attrValues //return value
		)
{
        char qrystr[500]; int swap=0;
	sprintf(qrystr, "select nodeID, attrValue from %s where graphID=%d and attrName=\'%s\' order by attrValue;", 
					tablename.c_str(), graphID, attrName.c_str()); 
	PGresult* res=queryCmd(qrystr);
	//cout<<"size:"<<PQntuples(res)<<endl;
	int j;
	// "j=1" means the range of attribute value starts from "1" and the maximum value: "2000", these two numbers (bounds) scan be changed according to different numerical values 
        for (j=0; j<=2000; j++){
		for(int i=0; i<PQntuples(res); i++)
		{
	        	char* iptr;
                	char* sptr;
                	int v1;
                	v1=atoi(PQgetvalue(res, i, 1));
                	if (v1==j) 
			{
                		iptr=PQgetvalue(res, i, 0);
				int v= (int) ntohl(*((uint32_t*)iptr));
               			v1=atoi(PQgetvalue(res, i, 1));
                		nodes.push_back(v);

				attrValues.push_back(v1);
			}
         	}
	}

       	PQclear(res);
}

//create the edge table
//schema: edge(graphID, node1ID, node2ID, attrName, attrType, attrValue)
//input: table name
void PGDB::createEdgeTable(string tablename)
{
	char qrystr[500];
	sprintf(qrystr, "create table %s(graphID int4, node1ID int4, node2ID int4,"
			"attrName varchar, attrType char(1),attrValue varchar)", 
			tablename.c_str());
	nonQueryCmd(qrystr);	
}

//insert one edge into the edge table
void PGDB::insertIntoEdgeTable(
		string tablename, 
		int graphID, 
		int node1ID, 
		int node2ID, 
		const Attributes& attrs //the attributes of the edge
)
{
	char qrystr[500];
	Attributes::const_iterator iter;
	for(iter=attrs.begin(); iter!=attrs.end(); iter++)
	{
		sprintf(qrystr, "insert into %s (graphID, node1ID, nodeID2, attrName, "
				"attrType, attrValue) values(%d, %d, %d, \'%s\', \'%c\', \'%s\')", 
				tablename.c_str(), graphID, node1ID, node2ID, iter->first.c_str(), 
				iter->second.type, iter->second.value.c_str());
		nonQueryCmd(qrystr);
	}
}

//create an index for getting edge attribute(s)
//edge_index(graphID, node1ID, node2ID, attrName, attrType, attrValue)
void PGDB::createEdgeIndex(string tablename)
{
	char qrystr[500];
	sprintf(qrystr, "create index %s_index on %s(graphID, node1ID, node2ID, "
			"attrName, attrType, attrValue)", tablename.c_str(), 
			tablename.c_str());
	nonQueryCmd(qrystr);
	sprintf(qrystr, "create index %s_index2 on %s(graphID, node2ID, node1ID)",
			tablename.c_str(), tablename.c_str());
	nonQueryCmd(qrystr);
}


//access one attribute of an edge
void PGDB::getEdgeAttr(
		string tablename, 
		int graphID, 
		int node1ID, 
		int node2ID, 
		string attrName,
		string& attrs //return value
)
{
	char qrystr[500];
	sprintf(qrystr, "select attrType, attrValue from %s where graphID=%d and "
			"node1ID=%d and node2ID=%d and attrName=\'%s\'", tablename.c_str(), 
			graphID, node1ID, node2ID, attrName.c_str());

	PGresult* res=queryCmd(qrystr);

	assert(PQnfields(res)==2);

	for(int i=0; i<PQntuples(res); i++)
	{
		char* ptr=PQgetvalue(res, i, 0);
		ptr=PQgetvalue(res, i, 1);
		attrs=string(ptr);
	}
	
	PQclear(res);
	
}

//access one attribute of an edge
void PGDB::getEdgeAttr(
		string tablename, 
		int graphID, 
		int node1ID, 
		int node2ID, 
		string attrName,
		Attributes& attrs //return value
)
{
	char qrystr[500];
	sprintf(qrystr, "select attrType, attrValue from %s where graphID=%d and "
			"node1ID=%d and node2ID=%d and attrName=\'%s\'", tablename.c_str(), 
			graphID, node1ID, node2ID, attrName.c_str());

	PGresult* res=queryCmd(qrystr);

	assert(PQnfields(res)==2);

	for(int i=0; i<PQntuples(res); i++)
	{
		AttributeValue attr;
		char* ptr=PQgetvalue(res, i, 0);
		attr.type=ptr[0];
		ptr=PQgetvalue(res, i, 1);
		attr.value=string(ptr);
		attrs.insert(Attributes::value_type(attrName, attr));
	}
	
	PQclear(res);
	
}

//access all the attributes of an edge
void PGDB::getEdgeAttr(
		string tablename, //input
		int graphID, //input
		int node1ID, //input
		int node2ID, //input
		Attributes& attrs //return value
)
{
	char qrystr[500];
	sprintf(qrystr, "select attrName, attrType, attrValue from %s where "
			"graphID=%d and node1ID=%d and node2ID=%d", tablename.c_str(),
			graphID, node1ID, node2ID);
	             
	PGresult* res=queryCmd(qrystr);
	
	assert(PQnfields(res)==3);

	for(int i=0; i<PQntuples(res); i++)
	{
		AttributeValue attr;
		char* ptr=PQgetvalue(res, i, 0);
		string attrName=string(ptr);
		ptr=PQgetvalue(res, i, 1);
		attr.type=ptr[0];
		ptr=PQgetvalue(res, i, 2);
		attr.value=string(ptr);
		attrs.insert(Attributes::value_type(attrName, attr));
	}
	
	PQclear(res);	
}

//select nodes based on the given condition
void PGDB::selectEdges(
		string tablename, //input
		string condition, //the input sql condition on the edge table
		vector<int>& graphs, //return value
		vector<int>& node1s, //return value
		vector<int>& node2s, //return
		vector<Attributes*>& vattrs	//return value
		)
{
	
	char qrystr[500];
	sprintf(qrystr, "select graphID, node1ID, node2ID, attrname, "
			"attrtype, attrvalue from %s %s order by graphID, node1ID, node2ID", 
			tablename.c_str(), condition.c_str());                                                                        
	PGresult* res=queryCmd(qrystr);

	if (PQntuples(res) != 0)          // if there is something wrong that causes the query result is empty, then do no execute 'storeEdges'
		storeEdges(res, graphs, node1s, node2s, vattrs);

	PQclear(res);
}

//store the edges in vectors from the database results
void PGDB::storeEdges(
		PGresult* res, //database resource
		vector<int>& graphs, //return value
		vector<int>& node1s, //return value
		vector<int>& node2s, //return
		vector<Attributes*>& vattrs	//return value
		)
{
	int pregid=0, prenid1=0, prenid2=0;
	Attributes* pattrs=NULL;
	for(int i=0; i<PQntuples(res); i++)
	{
		char* ptr=PQgetvalue(res, i, 0);
		int gid=(int) ntohl(*((uint32_t*)ptr));
		ptr=PQgetvalue(res, i, 1);
		int nid1=(int) ntohl(*((uint32_t*)ptr));
		ptr=PQgetvalue(res, i, 2);
		int nid2=(int) ntohl(*((uint32_t*)ptr));
		ptr=PQgetvalue(res, i, 3);
		string attrName=string(ptr);
		AttributeValue attr;
		ptr=PQgetvalue(res, i, 4);
		attr.type=ptr[0];
		ptr=PQgetvalue(res, i, 5);
		attr.value=string(ptr);
		debug(2, "graph: "<<gid<<" ("<<nid1<<", "<<nid2<<") attribute "<<attrName<<endl);
		if(gid!=pregid || prenid1!=nid1 || prenid2!=nid2)
		{
			debug(2, "insert \n");
			graphs.push_back(gid);
			node1s.push_back(nid1);
			node2s.push_back(nid2);
			pattrs=new Attributes;
			vattrs.push_back(pattrs);
			pregid=gid;
			prenid1=nid1;
			prenid2=nid2;
		}
		pattrs->insert(Attributes::value_type(attrName, attr));
	}
}

//select the imediate neighbors of a node based on the given condition
void PGDB::selectNeighbors(
		string tablename, //input
		int graphID, //input
		int nodeID, //input
		string condition, //the input sql condition on the edge table
		vector<int>& nodes, //return value
		string preparedplan //the name of a prepared plan
							//NULL if you don't want to use
							//prepared plan
)
{
	char qrystr[500];
	if(preparedplan.empty())
	{
		if(condition.empty())
		{
			sprintf(qrystr, "select nodeID from ( (select node2ID as nodeID "
				"from %s where graphID=%d and node1ID=%d) union "
				"(select node1ID as nodeID from %s where graphID=%d "
				"and node2ID=%d) ) t;", tablename.c_str(),
				graphID, nodeID, tablename.c_str(), graphID, 
				nodeID);
			
		}else
		{
			sprintf(qrystr, "select nodeID from ( (select node2ID as nodeID "
				"from %s where graphID=%d and node1ID=%d and %s) union "
				"(select node1ID as nodeID from %s where graphID=%d "
				"and node2ID=%d and %s) ) t;", tablename.c_str(),
				graphID, nodeID, condition.c_str(), tablename.c_str(), graphID, 
				nodeID, condition.c_str());
		}
	}else
	{
		sprintf(qrystr, "execute %s (%d, %d)", 
				preparedplan.c_str(), graphID, nodeID);
	}
	PGresult* res=queryCmd(qrystr);
	for(int i=0; i<PQntuples(res); i++)
	{
		char* iptr=PQgetvalue(res, i, 0);
		int ival=(int) ntohl(*((uint32_t*)iptr));
		nodes.push_back(ival);
	}
	
	PQclear(res);
}

//create the ortholog table
//schema: ortholog(orthologID, orthologName, source)
//input: table name
void PGDB::createOrthologTable(string tablename)
{
	char qrystr[500];
	sprintf(qrystr, "create table %s(orthologID int4, orthologName varchar,"
			" source varchar)", tablename.c_str());
	nonQueryCmd(qrystr);
}

//insert one ortholog into the ortholog table
void PGDB::insertIntoOrthologTable(
		string tablename, 
		int orthologID, 
		string orthologName, 
		string source
)
{
	char qrystr[500];
	sprintf(qrystr, "insert into %s (orthologID, orthologName, source)"
			" values(%d, \'%s\', \'%s\')", tablename.c_str(), 
			orthologID, orthologName.c_str(), source.c_str());
	nonQueryCmd(qrystr);
}

//create an index for getting ortholog
//ortholog_index(orthologID) on ortholog;
void PGDB::createOrthologIndex(string tablename)
{
	char qrystr[500];
	sprintf(qrystr, "create index %s_index on %s(orthologID)",
			tablename.c_str(), tablename.c_str());
	nonQueryCmd(qrystr);
}

//select the orthologName and source given a orthologID
void PGDB::getOrtholog(
		string tablename, //input
		int orthologID, //input
		string& orthologName, //return value 
		string& source, //return value
		string preparedplan //the name of a prepared plan
							//NULL if you don't want to use
							//prepared plan
)
{
	char qrystr[500];
	if(preparedplan.empty())
	{
		sprintf(qrystr, "select orthologName, source from %s where orthologID=%d",
				tablename.c_str(), orthologID);
	}else
	{
		sprintf(qrystr, "execute %s (%d)", preparedplan.c_str(), orthologID);
	}
	                                                                        
	PGresult* res=queryCmd(qrystr);
	
	assert(PQntuples(res)==1);
	assert(PQnfields(res)==2);

	char* ptr=PQgetvalue(res, 0, 0);
	orthologName=string(ptr);
	ptr=PQgetvalue(res, 0, 1);
	source=string(ptr);
	
	PQclear(res);
}

//create distance table
//distance(graphID, node1ID, node2ID, distance)
void PGDB::createDistanceTable(string tablename)
{
	char qrystr[500];
	sprintf(qrystr, "create table %s(graphID int4, node1ID int4, node2ID int4,"
			" distance int4);", tablename.c_str());
	nonQueryCmd(qrystr);
}

//insert one tuple into the distance table
void PGDB::insertIntoDistanceTable(
		string tablename, 
		int graphID, 
		int node1ID, 
		int node2ID, 
		int distance
		)
{
	char qrystr[500];
	sprintf(qrystr, "insert into %s (graphID, node1ID, node2ID, distance)"
			" values(%d, %d, %d, %d)", tablename.c_str(), 
			graphID, node1ID, node2ID, distance);
	nonQueryCmd(qrystr);
}

//create the distance index
//distance_index(graphID, node1ID, node2ID, distance) on distance
void PGDB::createDistanceIndex(string tablename)
{
	char qrystr[500];
	sprintf(qrystr, "create index %s_index on %s(graphID, node1ID, node2ID, distance)",
			tablename.c_str(), tablename.c_str());
	nonQueryCmd(qrystr);
}

//select the distance from node1 to node2 in the graph
int PGDB::selectDistance(
		string tablename, 
		int graphID, 
		int node1ID, 
		int node2ID,
		string preparedplan //the name of a prepared plan
							//NULL if you don't want to use
							//prepared plan
		)
{
	if(node1ID>node2ID)
		swap(node1ID, node2ID);
	char qrystr[500];
	if(preparedplan.empty())
	{
		sprintf(qrystr, "select distance from %s where graphID=%d and node1ID=%d "
				"and node2ID=%d", tablename.c_str(), graphID, node1ID, node2ID);
	}else
	{
		sprintf(qrystr, "execute %s (%d, %d, %d)", preparedplan.c_str(), graphID, 
				node1ID, node2ID);
	}
	debug(1, qrystr<<endl);                                                                        
	PGresult* res=queryCmd(qrystr);
	
	int distance=INT_MAX/10;
	if(PQntuples(res)==1)
	{
		assert(PQnfields(res)==1);
		char* iptr=PQgetvalue(res, 0, 0);
		distance=(int)ntohl(*((uint32_t*)iptr));
	}
	
	PQclear(res);
	                                                                        
	return distance;	
}

//select the nearby nodes with a certain distance with the node
//edge directions are ignored here 
void PGDB::selectNearbyNodes(
		string tablename, //input
		int graphID, //input
		int nodeID, //input
		int distance, //input
		vector<int>& nodes, //return value
		string preparedplan //the name of a prepared plan
							//NULL if you don't want to use
							//prepared plan
		)
{
	char qrystr[500];
	if(preparedplan.empty())
	{
		sprintf(qrystr, "select nodeID from (select nodeID2 as NodeID from %s where graphID=%d and "
				"node1ID=%d and distance=%d) union (select nodeID1 as NodeID from %s where graphID=%d "
				"and node2ID=%d and distance=%d) t", tablename.c_str(), graphID, 
				nodeID, distance, tablename.c_str(), graphID, nodeID, distance);
	}else
	{
		sprintf(qrystr, "execute %s (%d, %d, %d)", preparedplan.c_str(), graphID, 
				nodeID, distance);
	}
	                                                                        
	PGresult* res=queryCmd(qrystr);
	
	for(int i=0; i<PQntuples(res); i++)
	{
		char* iptr=PQgetvalue(res, i, 0);
		int ival=(int) ntohl(*((uint32_t*)iptr));
		nodes.push_back(ival);
	}
	
	PQclear(res);	
}

//create the fragment table
//fragment(orthSeq, distSum, graphID, nidSeq, distMatrix)
void PGDB::createFragmentTable(string tablename)
{
	char qrystr[500];
	sprintf(qrystr, "create table %s(orthSeq int8, distSum int4, graphID int4,"
			" nidSeq int4[], distMatrix int4[]);", tablename.c_str());
	nonQueryCmd(qrystr);	
}

//insert one tuple into the fragment table
void PGDB::insertIntoFragmentTable(
	string tablename, 
	int graphID, 
	const vector<int>& nodes,
	const vector<uint32_t>& distMatrix, 
	int summary, 
	const vector<int>& orthlogs
	)
{
	uint64_t orthSeq=getOrthSeq(orthlogs);
	uint32_t distSum;
	string matrixStr, nidSeq;
	getDistSeqSum(distMatrix, distSum, matrixStr);
	getNodeSeq(nodes, nidSeq);
	
	char qrystr[500];
	sprintf(qrystr, "insert into %s (orthSeq, distSum, graphID, nidSeq, distMatrix)"
			" values(%llu, %u, %d, \'%s\', \'%s\')", tablename.c_str(), orthSeq, distSum, graphID,
			nidSeq.c_str(), matrixStr.c_str());
	nonQueryCmd(qrystr);
}

//create the fragment index
//fragment_index(orthSeq, distSum) on fragment
void PGDB::createFragmentIndex(string tablename)
{
	char qrystr[500];
	sprintf(qrystr, "create index %s_index on %s(orthSeq, distSum)",
			tablename.c_str(), tablename.c_str());
	nonQueryCmd(qrystr);	
	
}

//create the clustered fragment index (fragment table is sorted)
//fragment_index(orthSeq, distSum) on fragment
void PGDB::createClusteredFragmentIndex(string tablename)
{
	char qrystr[500];
	sprintf(qrystr, "select * into %s_temp from %s order by orthSeq, distSum;", tablename.c_str(), tablename.c_str());
	nonQueryCmd(qrystr);
	sprintf(qrystr, "drop table %s;", tablename.c_str());
	nonQueryCmd(qrystr);
	sprintf(qrystr, "alter table %s_temp rename to %s;", tablename.c_str(), tablename.c_str());
	nonQueryCmd(qrystr);	
	sprintf(qrystr, "create index %s_index on %s(orthSeq, distSum)",
			tablename.c_str(), tablename.c_str());
	nonQueryCmd(qrystr);
}

//select fragments given the query fragment and the allowed the difference
void PGDB::selectFragments(
	string tablename, //input
	const uint64_t orthSeq, //input
	int distSumLower, //input
	int distSumUpper, //input
	vector<int>& graphs, //return value, the graphIDs of 
						//matching fragment
	vector<vector<int> >& fragments, //return value, the nodeIDs of 
									//the matching fragments
	vector<vector<int> >& matrices, //
	string preparedplan //the name of a prepared plan
						//NULL if you don't want to use
						//prepared plan
	)
{	
	char qrystr[500];
	if(preparedplan.empty())
	{
		sprintf(qrystr, "select graphID, nidSeq, distMatrix from %s where orthSeq=\'%llu\'"
				" and distSum>=%d and distSum<=%d;", tablename.c_str(), orthSeq, 
				distSumLower, distSumUpper);
	}else
	{
		sprintf(qrystr, "execute %s (%llu, %d, %d)", preparedplan.c_str(), orthSeq, 
				distSumLower, distSumUpper);
	}
	debug(1, qrystr<<endl);                        
	PGresult* res=queryCmd(qrystr);
	
	debug(1, "num results: "<<PQntuples(res)<<endl);
	int gid;
	for(int i=0; i<PQntuples(res); i++)
	{
		vector<int> fragment;
		vector<int> matrix;
		char* ptr=PQgetvalue(res, i, 0);
		gid=(int) ntohl(*((uint32_t*)ptr));
		ptr=PQgetvalue(res, i, 1);
		getIntVector(ptr, fragment);
		ptr=PQgetvalue(res, i, 2);
		getIntVector(ptr, matrix);
		graphs.push_back(gid);
		fragments.push_back(vector<int>(fragment));
		matrices.push_back(vector<int>(matrix));
	}
	PQclear(res);	
}

//create the neighborhood table
//neighborhood(orthologID, degree, nbConnection, nbArray, graphID, nodeID)
void PGDB::createNeighborhoodTable(string tablename)
{
	char qrystr[500];
	sprintf(qrystr, "create table %s(orthologID int4, degree int4, nbConnection int4, nbArray bit(96),"
			" graphID int4, nodeID int4);", tablename.c_str());
	nonQueryCmd(qrystr);
}

//insert one tuple into the neighborhood table
void PGDB::insertIntoNeighborhoodTable(
	string tablename, 
	int graphID, 
	int nodeID, 
	int orthologID, 
	int degree, 
	int nbConnection, 
	const vector<bool>& nbArray
	)
{
	char bitstr[97];//initialiezed in getBitString
	getBitString(nbArray, bitstr);
	bitstr[96]=0;
	
	char qrystr[1000];
	sprintf(qrystr, "insert into %s (orthologID, degree, nbConnection, nbArray, graphID, nodeID)"
			" values(%d, %d, %d, B\'%s\', %d, %d)", tablename.c_str(), orthologID, degree, nbConnection,
			bitstr, graphID, nodeID);
	nonQueryCmd(qrystr);
}

//create the neighborhood index
//neighborhood_index(orthologID, degree, nbConnection) on neighborhood
void PGDB::createNeighborhoodIndex(string tablename)
{
	char qrystr[500];
	sprintf(qrystr, "create index %s_index on %s(orthologID, degree, nbConnection)",
			tablename.c_str(), tablename.c_str());
	nonQueryCmd(qrystr);
}

//create the clustered neighborhood index (neighborhood table is sorted)
//neighborhood_index(orthologID, degree, nbConnection) on neighborhood	
void PGDB::createClusteredNeighborhoodIndex(string tablename)
{
	char qrystr[500];
	sprintf(qrystr, "select * into %s_temp from %s order by orthologID, degree, nbConnection;", 
			tablename.c_str(), tablename.c_str());
	nonQueryCmd(qrystr);
	sprintf(qrystr, "drop table %s;", tablename.c_str());
	nonQueryCmd(qrystr);
	sprintf(qrystr, "alter table %s_temp rename to %s;", tablename.c_str(), tablename.c_str());
	nonQueryCmd(qrystr);	
	sprintf(qrystr, "create index %s_index on %s(orthologID, degree, nbConnection)",
			tablename.c_str(), tablename.c_str());
	nonQueryCmd(qrystr);
}

//select the nodes that match the query neighborhood given the
//allowed the differences in number of matching neighbors and neighbor
//connections
void PGDB::selectNeighborhoods(
	string tablename, //input
	vector<int>& orths, //input
	int degree, //input
	int neighborDiff, //input
	int nbConnection, //input
	int connDiff, //input
	const bool* nbArray, //input
	vector<int>& graphs, //return value, the graphIDs of the 
						 //matching nodes 
	vector<int>& nodes, //return value, the nodeIDs of the
					   //matching nodes
	vector<float>& scores //return value, the score of each
						//mathing node
	)
{
	char qrystr[500];
	
	sprintf(qrystr, "select graphID, nodeID, nbArray, degree, nbConnection from %s where "
	"orthologID in (%s) and degree>=%d and nbconnection>=%d order by graphID, nodeID;", 
	tablename.c_str(), toString(orths, ',').c_str(), degree-neighborDiff, nbConnection-connDiff);
	                                                                        
	PGresult* res=queryCmd(qrystr);
	int gid, vid, degree_db, conn_db;
	float score;
	int array_size=96/8/sizeof(unsigned short);
	unsigned short array[array_size]; 
	for(int i=0; i<PQntuples(res); i++)
	{
		char* ptr=PQgetvalue(res, i, 2);
		for(int j=0; j<array_size; j++)
	            array[j]=(unsigned short) ntohs(*( (uint16_t*) (ptr+4+2*j)));
		//count the misses
        int miss=0;
        for(int k=0; k<96; k++)
        {
        	if(nbArray[k] && !isBitSet(array, k))
                miss++;
        }
        if(miss>neighborDiff)
            continue;
        
        ptr=PQgetvalue(res, i, 0);
        gid=(int) ntohl(*((uint32_t*)ptr));
        ptr=PQgetvalue(res, i, 1);
        vid=(int) ntohl(*((uint32_t*)ptr));
                                                                                                                                                     
        ptr=PQgetvalue(res, i, 3);
        degree_db=(int) ntohl(*((uint32_t*)ptr));
        ptr=PQgetvalue(res, i, 4);
        conn_db=(int) ntohl(*((uint32_t*)ptr));
        score=calScore(degree, degree_db, nbConnection, conn_db, miss);

		graphs.push_back(gid);
		nodes.push_back(vid);
		scores.push_back(score);
	}
	PQclear(res);	
}

//copy a table to a file using a specifed copy mode (e.g binary or text)
void PGDB::copyTableTo(
	string tablename, 
	string mode, 
	string path //the "absolute" path of the file
	)
{
	char qrystr[500];
	sprintf(qrystr, "copy %s %s to \'%s\' ;", mode.c_str(), tablename.c_str(), path.c_str());
	nonQueryCmd(qrystr);
}

//copy a file to a table using a specifed copy mode (e.g binary or text)
void PGDB::copyTableFrom(
	string tablename, 
	string mode, 
	string path //the "absolute" path of the file
	)
{
	char qrystr[500];
	sprintf(qrystr, "copy %s %s from \'%s\' ;", mode.c_str(), tablename.c_str(), path.c_str());
	nonQueryCmd(qrystr);
}

//prepare query plans, to avoid the overhead of finding query plans at
//query execution times
void PGDB::preparePlan(string plan)
{
	char qrystr[500];
	sprintf(qrystr, "%s;", plan.c_str());
	nonQueryCmd(qrystr);
}
