#ifndef GRAPH_H
#define GRAPH_H

#include <hash_map.h>
#include <assert.h>
#include <vector>
#include <string>
#include "../util/util.h"
#include "../db/db.h"

using namespace std;

typedef unsigned int UINT;

struct GraphEdge
{
	//bool outgoing;
	//bool incoming;
	Attributes* p_eattr;
};

typedef hash_map<UINT, GraphEdge> EdgeMap;

struct GraphNode
{
	Attributes nattr;
	EdgeMap* p_neighbors;
};

//right now assume graphs are directed.
class HashGraph
{
public:

	HashGraph(bool drct=true) 
	{ 
		//cout<<"in constructor\n"<<endl;
		gname="";
		numEdges = 0;
		diGraph=drct;
	}
	
	~HashGraph() 
	{ // Destructor
		EdgeMap::iterator p;
		for(unsigned int i=0; i<nodes.size(); i++)
		{
			for(p=nodes[i].p_neighbors->begin(); p!=nodes[i].p_neighbors->end(); p++)
			{
				if(i<=p->first)
					delete p->second.p_eattr;
				p->second.p_eattr=NULL;
			}
			delete nodes[i].p_neighbors;
			nodes[i].p_neighbors=NULL;
		}
	}
	
	UINT n() { return nodes.size(); } // Number of vertices
	UINT e() { return numEdges; }   // Number of edges
	
	Attributes* getGraphAttrs()
	{
		return graphAttrs;
	}
	
	void setGraphAttrs(Attributes* gatr)
	{
		graphAttrs=gatr;
	}
	// Set undirected edge (v1, v2) to wgt
	void setEdge(UINT v1, UINT v2, Attributes* patr) 
	{
		assert(v1<nodes.size());
		if(nodes[v1].p_neighbors->find(v2)!=nodes[v1].p_neighbors->end())
			return;
		//add edge (v1, v2)
		GraphEdge e1, e2;
		e1.p_eattr=e2.p_eattr=patr;
		nodes[v1].p_neighbors->insert(EdgeMap::value_type(v2, e1));
		//printf("edges\n");		
		if(v1!=v2)
		nodes[v2].p_neighbors->insert(EdgeMap::value_type(v1, e2));

		numEdges++;
	}
	// Set directed edge (v1, v2) to wgt
	void setDirectedEdge(UINT v1, UINT v2, Attributes* patr) 
	{
		assert(v1<nodes.size());
		if(nodes[v1].p_neighbors->find(v2)!=nodes[v1].p_neighbors->end())
			return;
		//add edge (v1, v2)
		GraphEdge e1, e2;
		e1.p_eattr=e2.p_eattr=patr;
		nodes[v1].p_neighbors->insert(EdgeMap::value_type(v2, e1));

		numEdges++;
	}

	Attributes* delEdge(UINT v1, UINT v2)
	{
		assert(v1<nodes.size());
		EdgeMap::iterator p=nodes[v1].p_neighbors->find(v2);
		if(p==nodes[v1].p_neighbors->end())
			return NULL;
		Attributes* pattr=p->second.p_eattr;
		nodes[v1].p_neighbors->erase(p);
		if(v1!=v2)
			nodes[v2].p_neighbors->erase(v1);
		numEdges--;
		return pattr;
	}
	
	EdgeMap* getNeighbors(UINT v)
	{
		assert(v<nodes.size());
		return nodes[v].p_neighbors;
	}
	bool isEdge(UINT v1, UINT v2)
	{
		assert(v1<nodes.size());
		EdgeMap::iterator p=nodes[v1].p_neighbors->find(v2);
		if(p==nodes[v1].p_neighbors->end())
			return false;
		return true;
	}
	Attributes* getEdgeAttrs(UINT v1, UINT v2)
	{
		assert(v1<nodes.size());
		EdgeMap::iterator p=nodes[v1].p_neighbors->find(v2);
		if(p==nodes[v1].p_neighbors->end())
			return NULL;
		return p->second.p_eattr;
	}
	Attributes* getNodeAttrs(UINT v)
	{
		assert(v<nodes.size());
		return &(nodes[v].nattr);
	}
	void setNodeAttrs(UINT v, Attributes* pnattr)
	{
		assert(v<nodes.size());
		nodes[v].nattr=*pnattr;
	}
	int degree(UINT v)
	{
		assert(v<nodes.size());
		return nodes[v].p_neighbors->size();
	}
	UINT addNode(Attributes* pnattr)
	{
		UINT v=nodes.size();
		GraphNode nd;
		nd.nattr=*pnattr;
		nd.p_neighbors= new EdgeMap;
		nodes.push_back(nd);
		return v;
	}
	//void setName(const string& name){ gname=name;}
	//const string& getName() {return gname;}
	
private:
	int numEdges;     // Number of vertices, edges
	vector<GraphNode> nodes; //pointer to the node array
	bool diGraph;		//whether the graph is directed
	string gname;
	Attributes* graphAttrs;
};

#endif
