#include "matchio.h"
#include "../util/graphutil.h"

void writeMatch(
		ostream& s, 
		GraphMatch& M
		)
{
	NodeMappingSet::iterator iter;
	s<<"---------------------------------------------"<<endl;
	s<<"Database Match Graph ID: "<<M.dbgid<<endl;
	s<<"Match Score: "<<M.score<<endl;
	for(iter=M.mappings.begin(); iter!=M.mappings.end(); iter++)
         s<<iter->source<<"<->"<<iter->target<<"\n";
}

void writeMatchGRF(
		ostream& s, 
		HashGraph* G1, 
		OrthologInfoList* pOrthinfolist1, 
		HashGraph* G2, 
		OrthologInfoList* pOrthinfolist2, 
		GraphMatch& M, 
		Ortholog& orth
		)
{
	s<<G1->getGraphAttrs()->find("name")->second.value<<"_vs_"<<G2->getGraphAttrs()->find("name")->second.value<<endl;
	s<<M.mappings.size()<<endl;
	NodeMappingSet::iterator iter, iter2;
	for(iter=M.mappings.begin(); iter!=M.mappings.end(); iter++)
    {
         s<<G1->getNodeAttrs(iter->source)->find("name")->second.value<<"<->"<<G2->getNodeAttrs(iter->target)->find("name")->second.value<<"\t";

		//find common orthlogs
		vector<int> commonOrth;
		for(unsigned int i=0; i<(*pOrthinfolist1)[iter->source].size(); i++)
		{
			for(unsigned int t=0; t<(*pOrthinfolist2)[iter->target].size(); t++)
				if((*pOrthinfolist1)[iter->source][i]==(*pOrthinfolist2)[iter->target][t])
				{
					commonOrth.push_back((*pOrthinfolist1)[iter->source][i]);
					break;
				}
		}
	    s<<commonOrth.size();
			 
		for(unsigned int i=0; i<commonOrth.size(); i++)
			s<<"\t"<<orth.getOrthologName(commonOrth[i]);
		s<<endl;
    }
	
	vector<int> sources;
	vector<int> targets;
	int  i, j;
	i=0;
	for(iter=M.mappings.begin(); iter!=M.mappings.end(); iter++)
	{
		iter2=iter;
		iter2++;
		j=i+1;
		for(;iter2!=M.mappings.end(); iter2++)
		{
			bool q_direct= ( G1->isEdge(iter->source, iter2->source) );
			bool db_direct= ( G2->isEdge(iter->target, iter2->target) );
			if( q_direct && db_direct)
			{
				sources.push_back(i);
				targets.push_back(j);
			}
			j++;
		}
		i++;
	}
	s<<sources.size()<<endl;

	for(unsigned int i=0; i<sources.size(); i++)
		s<<sources[i]<<"\t"<<targets[i]<<"\t1.0\n";

}

void writeMatchGML(
		ostream& s, 
		HashGraph* G1, 
		OrthologInfoList* pOrthinfolist1, 
		HashGraph* G2, 
		OrthologInfoList* pOrthinfolist2, 
		GraphMatch& M
		)
{
	hash_set<int> qnodes, dbnodes;
	//debug(37, "numEdges = "<<Gdb->e()<<endl);
	s<<"graph [\n\tdirected 0";
	
	int  i, j;
	i=0;
	NodeMappingSet::iterator iter, iter2;
	for(iter=M.mappings.begin(); iter!=M.mappings.end(); iter++)
	{
		s<<"\n\tnode [\n\t\tid "<<i<<"\n\t\tlabel \""<<G1->getNodeAttrs(iter->source)->find("name")->second.value
			<<"<->"<<G2->getNodeAttrs(iter->source)->find("name")->second.value<<"\"";
		s<<"\n\t\tgraphics [\n\t\t\ttype \"rectangle\"\n\t\t\tfill \"#00ccff\"\n\t\t]\n\t]";
		i++;
	}

	i=0;
	for(iter=M.mappings.begin(); iter!=M.mappings.end(); iter++)
	{
		iter2=iter;
		iter2++;
		j=i+1;
		for(;iter2!=M.mappings.end(); iter2++)
		{
		      bool q_direct= ( G1->isEdge(iter->source, iter2->source) );
		      bool db_direct= ( G2->isEdge(iter->target, iter2->target) );

			if(!q_direct && !db_direct)
				continue;
			else
			{
				s<<"\n\tedge [\n\t\tsource "<<i<<"\n\t\ttarget "<<j;
				if( q_direct && db_direct)
				{
					s<<"\n\t\tlabel \"both\"\n\t\tgraphics [\n\t\t\ttype \"line\"\n\t\t\twidth 8\n\t\t\tfill \"#0000ff\"\n\t\t]\n\t]";
				}
				else if(q_direct)
				{
					s<<"\n\t\tlabel \"query\"\n\t\tgraphics [\n\t\t\ttype \"line\"\n\t\t\twidth 1\n\t\t\tfill \"#ff9933\"\n\t\t]\n\t]";
				}
				else
				{
					s<<"\n\t\tlabel \"database\"\n\t\tgraphics [\n\t\t\ttype \"line\"\n\t\t\twidth 1\n\t\t\tfill \"#339900\"\n\t\t]\n\t]";
				}
			}
			j++;
		}
		i++;
	}
	s<<"\n]";
}

void writeMatchDOT(
		ostream & s, 
		HashGraph * G1, 
		OrthologInfoList* pOrthinfolist1, 
		int** D1, 
		HashGraph * G2, 
		OrthologInfoList* pOrthinfolist2, 
		int** D2, 
		GraphMatch & M
		)
{
	s<<"graph G {\n";
	//the query graph
	s<<"\tsubgraph cluster0 {\n";
	s<<"\t\tlabel=\"query: "<<G1->getGraphAttrs()->find("name")->second.value<<"\";\n";
	s<<"\t\tcolor=blue;\n";
	
	NodeMappingSet::iterator iter, iter1, iter2;
	
	hash_set<int> qnmset, dbnmset;
	
	for(iter=M.mappings.begin(); iter!=M.mappings.end(); iter++)
	{
		qnmset.insert(iter->source);
		dbnmset.insert(iter->target);
	}
	
	for(int i=0; i<G1->n(); i++)
	{
		if(qnmset.find(i)!=qnmset.end())
			s<< "\t\tq"<<i<<" [label=\""<<G1->getNodeAttrs(i)->find("name")->second.value<<"\" shape=box style=filled fillcolor=lightblue color=blue];\n";
		else
			s<< "\t\tq"<<i<<" [label=\""<<G1->getNodeAttrs(i)->find("name")->second.value<<"\" shape=box];\n";
	}
	
	char arrow[3];
	strcpy(arrow, "--");
	
	//non-important edges
	for(int i=0; i<G1->n(); i++)
		for(int j=i+1; j<G1->n(); j++)
		{
			if(G1->isEdge(i,j) && (qnmset.find(i)==qnmset.end() || qnmset.find(j)==qnmset.end()) )
			{
				s<< "\t\tq"<<i<<" "<<arrow<<" q"<<j<<";\n";
			}
		}
	
	//important edges
	for(iter1=M.mappings.begin(); iter1!=M.mappings.end(); iter1++)
	{
		iter2=iter1;
		iter2++;
		for(; iter2!=M.mappings.end(); iter2++)
		{
			if(G1->isEdge(iter1->source, iter2->source))
			{ 
				if(G2->isEdge(iter1->target, iter2->target))
					s<< "\t\tq"<<iter1->source<<" "<<arrow<<" q"<<iter2->source<<" [style=\"bold\" color=blue];\n";
				else
					s<< "\t\tq"<<iter1->source<<" "<<arrow<<" q"<<iter2->source<<";\n";
			}
		}
	}
	
	s<< "\t}\n";
	//the database graph
		
	s<< "\tsubgraph cluster1 {\n";
	
	s<< "\t\tlabel=\"DB graph: "<<G2->getGraphAttrs()->find("name")->second.value<<"\";\n";
	
	s<< "\t\tcolor=blue;\n";
	
	set<int> vset;
	set<int>::iterator p1, p2;
	for(iter1=M.mappings.begin(); iter1!=M.mappings.end(); iter1++)
	{
		iter2=iter1;
		iter2++;
		for(; iter2!=M.mappings.end(); iter2++)
		{
			int dist=D2[iter1->target][iter2->target];
			getBridge(G2, iter1->target, iter2->target, dist, vset);
		}
	}
	
	set<int> another(vset);

	//db graph vertices
	vector<string> nodes;
	for(iter=M.mappings.begin(); iter!=M.mappings.end(); iter++)
	{
		another.erase(iter->target);
		s<< "\t\tdb"<<iter->target<<" [label=\""<<G2->getNodeAttrs(iter->source)->find("name")->second.value
			<<"\" shape=box style=filled fillcolor=lightblue color=blue];\n";
			
	}

		
	for(p1=another.begin(); p1!=another.end(); p1++)
	{
		s<< "\t\tdb"<< (*p1)<<" [label=\""<<G2->getNodeAttrs(iter->source)->find("name")->second.value<<"\" shape=box];\n";
	}
	//non important edges
	for(p1=vset.begin(); p1!=vset.end(); p1++)
	{
		p2=p1;
		p2++;
		for(; p2!=vset.end(); p2++)
		{
			if( (dbnmset.find((*p1))==dbnmset.end() || dbnmset.find((*p2))==dbnmset.end()) && G2->isEdge((*p1), (*p2)))
			{
				
				s<< "\t\tdb"<<(*p1)<<" "<<arrow<<" db"<<(*p2)<<";\n";
			}
		}
	}
	
	
	//important edges
	for(iter1=M.mappings.begin(); iter1!=M.mappings.end(); iter1++)
	{
		iter2=iter1;
		iter2++;
		for(; iter2!=M.mappings.end(); iter2++)
		{
			if(G2->isEdge(iter1->target, iter2->target))
			{
				if(G1->isEdge(iter1->source,iter2->source))
				{
					s<< "\t\tdb"<<iter1->target<<" "<<arrow<<" db"<<iter2->target<<" [style=\"bold\" color=blue];\n";
				}
				else
				{
					s<< "\t\tdb"<<iter1->target<<" "<<arrow<<" db"<<iter2->target<<";\n";
				}
			}
		}
	}
	
	s<< "\t}\n";

	//emphasize the mapping
	for(iter=M.mappings.begin(); iter!=M.mappings.end(); iter++)
		s<< "\tq"<<iter->source<<" -- db"<<iter->target<<" [color=red]\n";
	s<< "}\n";	
}

void writeMatchGDF(
		ostream& s, 
		HashGraph* G1, 
		OrthologInfoList* pOrthinfolist1, 
		HashGraph* G2, 
		OrthologInfoList* pOrthinfolist2, 
		GraphMatch& M, 
		Ortholog& orth
		)
{
	NodeMappingSet::iterator iter, iter2;
	//write the header for nodes
	s<<"nodedef>name,orthologies varchar\n";
	for(iter=M.mappings.begin(); iter!=M.mappings.end(); iter++)
    {
		string name=G1->getNodeAttrs(iter->source)->find("name")->second.value
		+"<->"+G2->getNodeAttrs(iter->target)->find("name")->second.value;
		replaceBadChar(name, ",", '|');
		s<<name;

		//find common orthlogs
		vector<string> commonOrth;
		for(unsigned int i=0; i<(*pOrthinfolist1)[iter->source].size(); i++)
		{
			for(unsigned int t=0; t<(*pOrthinfolist2)[iter->target].size(); t++)
				if((*pOrthinfolist1)[iter->source][i]==(*pOrthinfolist2)[iter->target][t])
				{
					commonOrth.push_back( orth.getOrthologName( (*pOrthinfolist1)[iter->source][i] ) );
					break;
				}
		}
		string orthstr=toString(commonOrth, ':');
		replaceBadChar(orthstr, ",", '|');
		s<<","<<orthstr<<endl;
    }
	
	//write header
	s<<"edgedef>node1,node2\n";
	for(iter=M.mappings.begin(); iter!=M.mappings.end(); iter++)
	{
		iter2=iter;
		iter2++;
		for(;iter2!=M.mappings.end(); iter2++)
		{
			bool q_direct= ( G1->isEdge(iter->source, iter2->source) );
			bool db_direct= ( G2->isEdge(iter->target, iter2->target) );
			if( q_direct && db_direct)
			{
				string name1=G1->getNodeAttrs(iter->source)->find("name")->second.value+
				"<->"+G2->getNodeAttrs(iter->target)->find("name")->second.value;
				replaceBadChar(name1, ",", '|');
				s<<name1;
				string name2=G1->getNodeAttrs(iter2->source)->find("name")->second.value
				+"<->"+G2->getNodeAttrs(iter2->target)->find("name")->second.value;
				replaceBadChar(name2, ",", '|');
				s<<","<<name2<<endl;
			}
		}
	}
}
