#include "summaryio.h"

float writeSummaryGDFFormat(ostream& s, ColorArray& colorArr, string attrname, char attrtype, vector<string>& attrvalues, string edgetype, bool directedGraph, int& num_strong_rel)
{
	s<<"nodedef>name,numNodes INT";
	writeGDFAttribute(s, attrname, attrtype);
	s<<endl;
        float coverage=0; 			// the coverage
	int diversity=0;  			// the diversity
	vector<struct colorPair> attr;          // the vector to store the pair of different categorical attribute values

	ColorArray::iterator it;
	int graphsize=0;
	for(it=colorArr.begin(); it!=colorArr.end(); it++)
	{	
		s<<"n"<<it->first<<","<<it->second->getSize()<<","<<attrvalues[it->second->initialColor]<<endl;
		graphsize += it->second->getSize();
	}	
	s<<"edgedef>node1,node2,participation FLOAT,directed,type varchar\n";
	
	ColorArray::iterator p;
	hash_map<UINT, int>::iterator q;
	for(p=colorArr.begin(); p!=colorArr.end(); p++)
	{
		UINT color1=p->first;
		int total1=p->second->getSize();
		for(q=p->second->participates.begin(); q!=p->second->participates.end(); q++)
		{
			if (!directedGraph) {
				UINT color2=q->first;
				if(q->second==0 || color1>color2)
				continue;
				
				int part1=q->second;
				int part2=colorArr[color2]->getParticipates(color1);

				assert(part2>0);
				int total2=colorArr[color2]->getSize();
				
				int part=part1+part2;

				int total=total1+total2;
				float support = (float)total/graphsize;
				float weight = (float)part/(float)total;
	                        if ( weight >= 0.5 && color1 !=color2)
			 	{
					struct colorPair tmp;
					tmp.color1 = p->second->initialColor;
					tmp.color2 = colorArr[color2]->initialColor;
						
					if (attr.size()==0)             		      // if the vector is empty, directly add the pair of different attribute values into it
					{
						attr.push_back(tmp);
						diversity++;
					}
					else
					{
						int it=0;
						int found=0;
						for (it=0;it<attr.size(); it++)               // check if the pair of different attribute values has appeared before
						{
						if((attr[it].color1 == tmp.color1 && attr[it].color2 == tmp.color2) || (attr[it].color2 == tmp.color1 && attr[it].color1 == tmp.color2))
								found=1;
						}
						if (found == 0)  			      // if the pair has not been found, add it to the vector
						{
							if (tmp.color1 != tmp.color2)
							{
								attr.push_back(tmp);
								diversity++;                  // increase diversity by 1
							}
						}
					}
	                        coverage += weight * support;	                              // calculate the coverage
				num_strong_rel++;				              // number of strong relationship ++ 	
				}
			
			s<<"n"<<color1<<",n"<<color2<<","<<(float)part1/(float)total1<<","<<"false"<<","<<edgetype<<endl;
			}
			else
			{
				UINT color2=q->first;
				if(q->second==0)
					continue;
				
				int part1=q->second;

				int total2=colorArr[color2]->getSize();

				int total=total1+total2;
				float support = (float)total/graphsize;
				float weight = (float)part1/(float)total1;

	                        if ( weight >= 0.5 && color1 !=color2)
			 	{
					struct colorPair tmp;
					tmp.color1 = p->second->initialColor;
					tmp.color2 = colorArr[color2]->initialColor;
						
					if (attr.size()==0)
					{
						attr.push_back(tmp);
						diversity++;
					}
					else
					{
						int it=0;
						int found=0;
						for (it=0;it<attr.size(); it++)
						{
							if(attr[it].color1 == tmp.color1 && attr[it].color2 == tmp.color2)
								found=1;
						}
						if (found == 0)
						{
							if (tmp.color1 != tmp.color2)
							{
								attr.push_back(tmp);
								diversity++;
							}
						}
					}
		                        coverage += weight * support;	
					num_strong_rel++;
				}

				s<<"n"<<color1<<",n"<<color2<<","<<(float)part1/(float)total1<<","<<"true"<<","<<edgetype<<endl;
			}
		}
	}

return coverage * float(diversity)/float(attrvalues.size());                          // return the value of diversity * coverage
}

