/******************************************************************************
 * The declarition of class LoadGraph which loads the graph into database. 
 * Author: Yuanyuan Tian (ytian@umich.edu)
 * ***************************************************************************/

#include "../db/db.h"
#include "../db/pgsql_db.h"
#include "../orthology/ortholog.h"
#include "../util/graphutil.h"
#include "../util/matrix.h"


class LoadGraph
{
public:
	LoadGraph(int fz, int flimt, DB* db, WriteToFile* wf, string prefix, int direct)
	{	
		fragSize=fz;
		fragLimit=flimt;
		this->db=db;
		wtf=wf;
		tablePrefix=prefix;
		directional=direct;
	}
	void createTables()
	{
		db->createGraphTable(tablePrefix+"_graph");
		db->createNodeTable(tablePrefix+"_node");
		db->createEdgeTable(tablePrefix+"_edge");
		db->createOrthologTable(tablePrefix+"_orthology");
		db->createDistanceTable(tablePrefix+"_distance");
		db->createFragmentTable(tablePrefix+"_fragment");
		db->createNeighborhoodTable(tablePrefix+"_neighborhood");
		
	}

	void copyData(string path)
	{
		//db->copyTableFrom(tablePrefix+"_graph", "binary", path+"graph.bin");
		db->copyTableFrom(tablePrefix+"_node", "binary", path+"node.bin");
		db->copyTableFrom(tablePrefix+"_edge", "binary", path+"edge.bin");
		db->copyTableFrom(tablePrefix+"_orthology", "binary", path+"orth.bin");
		db->copyTableFrom(tablePrefix+"_distance", "binary", path+"dist.bin");
		db->copyTableFrom(tablePrefix+"_fragment", "binary", path+"frag.bin");
		db->copyTableFrom(tablePrefix+"_neighborhood", "binary", path+"neig.bin");
	}
	
	void createIndices()
	{
		db->createGraphIndex(tablePrefix+"_graph");
		db->createNodeIndex(tablePrefix+"_node");
		db->createEdgeIndex(tablePrefix+"_edge");
		db->createOrthologIndex(tablePrefix+"_orthology");
		db->createDistanceIndex(tablePrefix+"_distance");
		db->createClusteredFragmentIndex(tablePrefix+"_fragment");//Note: clustered index
		db->createClusteredNeighborhoodIndex(tablePrefix+"_neighborhood");//Note: clustered index
	}
	
	void setGraph(int id, HashGraph* grf, OrthologInfoList* oinfo, Ortholog* pot)
	{
		gid=id;
		G=grf;
		pOrthinfolist=oinfo;
		pOrth=pot;
	}
	
	void load(bool forsaga=true, bool fortale=true);
	
private:
	
	void insertGraph();
	void insertNodes();
	void insertEdges();
	void insertDistances();
	void insertFragments();
	void insertNeighborhoods();
	void insertOrthologs();

	void insertFragmentHelp( NodeSet & vs, unsigned int iterIndx, vector < NodeOrtholog > & vfset);
	void insertFragmentTuple( vector < NodeOrtholog > & vfset);
	
	int fragSize;
	int fragLimit;
	DB* db;
	WriteToFile* wtf;
	string tablePrefix;
	int directional;
	int** D;
	HashGraph* G;
	OrthologInfoList* pOrthinfolist;
	Ortholog* pOrth;
	int gid;
};
