/******************************************************************************
 * The declaration of fuctions and types related to the orthology information
 * of nodes. 
 * Author: Yuanyuan Tian (ytian@umich.edu)
 * ***************************************************************************/

#ifndef ORTHOLOG_H
#define ORTHOLOG_H
#include <hash_map.h>
#include "../util/util.h"
using namespace std;

//the hash map used to store the ortholog name to ortholog id mapping
typedef hash_map<string, uint32_t, strhash, eqstr> OrthologName2IDMap;

//the hash map used to store the ortholog id to ortholog name mapping
typedef hash_map<uint32_t, string, inthash> OrthologID2NameMap;

class Ortholog
{
public:
	//the function that takes the file of ortholog names and build
	//the ortholog hash map
	void buildOrthologMap(FILE* f);
	
	//given a ortholog name key, find the ortholog id
	uint32_t getOrthologID(string name);
	
	string getOrthologName(uint32_t id);
	
	OrthologName2IDMap* getName2IDMap()
	{
		return &n2idmap;
	}
	
	OrthologID2NameMap* getID2NameMap()
	{
		return &id2nmap;
	}
	
private:	
	//the hash map used to store the ortholog name to ortholog id mapping
	OrthologName2IDMap n2idmap;
	OrthologID2NameMap id2nmap;
};

//*******************************************
// type for a vertex and its associated orthology id
//*******************************************
struct NodeOrtholog
{
	int nodeID;//node id
	int orthID;//ortholog id
};

//the less than comparison of Vertex_Family based on the family id
//used to group all the probes by the fid label
bool ltNO(const NodeOrtholog vo1, const NodeOrtholog vo2);

typedef vector<int> OrthologInfo;
typedef vector<OrthologInfo> OrthologInfoList;

typedef vector<NodeOrtholog> NodeOrthologList;
#endif

