#include "snap/summarizer.h"
#include "visual/visual.h"
#include "graph/hashgraph.h"
#include "config.h"
#include "db/db.h"
#include "snap/color.h"
#include "db/pgsql_db.h"
#include "util/graphutil.h"
#include "io/summaryio.h"
#include "util/mytimer.h"

int main(int argc, char** argv) 
{
	
	if (argc !=8 ) 
	{
	        cerr << "Usage: querysnap [configuration file][table Prefix] [graph id] [attrName] [edgeType] [k: resolution] [file prefix]\n";
	        exit(-1);
        }
	
	Config cf;
    	cf.readFile(argv[1]);
   	// cf.printConfig();
	string path=string(argv[7]);
	if(path[path.size()-1]!='/')
		path=path+"/";
    
   	 PGDB* db=new PGDB((char*)cf.dbname.c_str(), (char*)cf.dbuser.c_str(), (char*)cf.dbpwd.c_str());
	
   	string tablePrefix=string(argv[2]);
	int g=atoi(argv[3]);
	string attrname=string(argv[4]);
	string edgetype=string(argv[5]);
	float interesting;
	//////////////////////////////////////
	Summarizer smr(db);
	
	//set the graph attributes and load the graph into memory
	smr.setGraph(tablePrefix, g, attrname, edgetype);

	MicroTimer timer;
        timer.reset();
	debug(33, "set graph succeed\n");
	
	int k=atoi(argv[6]);
	if(k<=0)
	{
		cerr<<"Error: in valid k value (k>0): "<<k<<endl;
		exit(-1);
	}
	cout<<"-- Resolution: "<<k<<endl;

	smr.kSNAP(k);                   //top-down k-SNAP operation
	
	debug(33, "ksnap succeed\n");
	char filename[200];
	sprintf(filename, "%s%s_summary_%d.gdf", path.c_str(), tablePrefix.c_str(), k);
	cout<<"-- Time Spent Totally (s): "<<timer.stop_seconds()<<endl;
	ofstream of(filename);
	if(!of.is_open())
	{
		cout<<"Error: Can not open file "<<filename<<endl;
		exit(-1);
	}
	int num_strong_rel=0;
	
	// save the graph summarization into GDF format and compute the value of (diversity * coverage) 
	interesting = writeSummaryGDFFormat(of, smr.colorArr, attrname, 's', Color::attrValues, edgetype, smr.directedGraph, num_strong_rel);

	cout<<"-- Interestingness Score --"<< interesting/(float)(num_strong_rel+k)<<endl;
	
	of.close();

	delete db;
	return 0;
}
