#ifndef MINHEAP_H
#define MINHEAP_H
#include <assert.h>
#define key(x) x.key
#define value(x) x.value

//Things that user has to define before using this class
//(1) types KEY, VALUE
//(2) <, ==, <=, >, >= operations on KEY
//(3) cout<< functions for KEY and VALUE types
//(4) hash function ValueHash for VALUE
//(5) equal_to function ValueEqual for VALUE

template < class KEY, class VALUE>
struct ELEM
{
	KEY key;
	VALUE value;
};

template < class KEY, class VALUE, class ValueHash, class ValueEqual >
class MinHeap {                 // Min-heap class
private:
  vector< ELEM<KEY, VALUE> >* Heap;                // Pointer to the heap array
  int size;                  // Maximum size of the heap
  int n;// Number of elements now in the heap
  void siftDown(int);        // Put an element in its correct place
public:
  hash_map<VALUE, int, ValueHash, ValueEqual>* refmap;
  MinHeap(vector<ELEM<KEY, VALUE> >* h, int num, int max);     // Constructor
  int heapSize() const;      // Return current size of the heap
  bool isLeaf(int) const;    // TRUE if pos is a leaf position
  int leftChild(int) const;  // Return position for left child
  int rightChild(int) const; // Return position for right child
  int parent(int) const;     // Return position for parent of pos
  void insert(const ELEM<KEY, VALUE>);   // Insert value into heap
  ELEM<KEY, VALUE> removeMin();          // Remove minimum value
  ELEM<KEY, VALUE> remove(int);          // Remove value at specified position
  void buildHeap();			// Heapify contents of Hea
  ELEM<KEY, VALUE> getMin(){return (*Heap)[0];}; //get the min value
  ELEM<KEY, VALUE> replaceMin(ELEM<KEY, VALUE> x); //replace the min value with x
  bool isEmpty(){ return (n==0);};// TRUE if heap is empty
  // reset the length of heap
  void resetLen( int length ){ assert(length<=size); n=length;};
  void Swap( int pos1, int pos2 );
  void changeKey(ELEM<KEY, VALUE> e);
  void removeKey(VALUE value);
  ~MinHeap();
  void printHeap();
  void printRefMap();
};

template < class KEY, class VALUE, class ValueHash, class ValueEqual >
void MinHeap< KEY, VALUE, ValueHash, ValueEqual>::printHeap()
{
	for(int i=0; i<n; i++)
	{
		cout<<(key((*Heap)[i]));
		cout<<" -- ";
		cout<<(value((*Heap)[i]));
		cout<<endl;
	}
}
template < class KEY, class VALUE, class ValueHash, class ValueEqual >
void MinHeap< KEY, VALUE, ValueHash, ValueEqual>::printRefMap()
{
	hash_map<VALUE, int, ValueHash, ValueEqual>::iterator p;
	for(p=refmap->begin(); p!=refmap->end(); p++)
	{
		if(p->second>=0)
		{
			cout<<(p->first);
			cout<<" ~~ "<<p->second<<endl;
		}
	}
}
template < class KEY, class VALUE, class ValueHash, class ValueEqual >
void MinHeap< KEY, VALUE, ValueHash, ValueEqual>::changeKey(ELEM<KEY, VALUE> e)
{
	int oldpos=(*refmap)[e.value];
	assert(oldpos>=0 && oldpos<n);
	/*if(oldpos>=n || oldpos<0)
	{
		cout<<"in changeKey"<<endl;
		cout<<e.value<<" -- "<<e.key<<endl;
		cout<<"oldpos = "<<oldpos<<endl;
		printHeap();
		printRefMap();
		exit(-1);
	}*/
	remove(oldpos);
	insert(e);
}
template < class KEY, class VALUE, class ValueHash, class ValueEqual >
void MinHeap< KEY, VALUE, ValueHash, ValueEqual>::removeKey(VALUE value)
{
	hash_map<VALUE, int, ValueHash, ValueEqual>::iterator p=refmap->find(value);
	if(p==refmap->end()) return;
	int oldpos=p->second;

	if(oldpos<0)
		return;

	assert(oldpos>=0 && oldpos<n);
	remove(oldpos);
	p->second=-1;
}
template < class KEY, class VALUE, class ValueHash, class ValueEqual >
void MinHeap< KEY, VALUE, ValueHash, ValueEqual>::Swap( int pos1, int pos2 )
{
	(*refmap)[(*Heap)[pos1].value]=pos2;
	(*refmap)[(*Heap)[pos2].value]=pos1;
	
	ELEM<KEY, VALUE> e=(*Heap)[pos1];
	(*Heap)[pos1]=(*Heap)[pos2];
	(*Heap)[pos2]=e;
}
template < class KEY, class VALUE, class ValueHash, class ValueEqual >
MinHeap< KEY, VALUE, ValueHash, ValueEqual>::MinHeap(vector<ELEM<KEY, VALUE> >* h, int num, int max)   // Constructor
{
	Heap = h;  
	n = num;  
	size = max;
	refmap=new hash_map<VALUE, int, ValueHash, ValueEqual>();
	for(int i=0; i<n; i++)
	{
		refmap->insert(hash_map<VALUE, int, ValueHash, ValueEqual>::value_type((*Heap)[i].value, i));
	}
	buildHeap(); 
}
template < class KEY, class VALUE, class ValueHash, class ValueEqual >
MinHeap< KEY, VALUE, ValueHash, ValueEqual>::~MinHeap()
{
	delete refmap;
}
//****************************************************************

template < class KEY, class VALUE, class ValueHash, class ValueEqual >
int MinHeap< KEY, VALUE, ValueHash, ValueEqual>::heapSize() const // Return current size of the heap
{ return n; }
//****************************************************************

template < class KEY, class VALUE, class ValueHash, class ValueEqual >
bool MinHeap< KEY, VALUE, ValueHash, ValueEqual>::isLeaf(int pos) const // TRUE if pos is a leaf position
  { return (pos >= n/2) && (pos < n); }
//***************************************************************

// Return position for left child of pos
template < class KEY, class VALUE, class ValueHash, class ValueEqual >
int MinHeap< KEY, VALUE, ValueHash, ValueEqual>::leftChild(int pos) const {
  assert(pos < n/2);  // Must be a position with a left child
  return 2*pos + 1;
}
//******************************************************************

// Return position for right child of pos
template < class KEY, class VALUE, class ValueHash, class ValueEqual >
int MinHeap< KEY, VALUE, ValueHash, ValueEqual>::rightChild(int pos) const {
  assert(pos < (n-1)/2);  // Must be a position with a left child
  return 2*pos + 2;
}
//*****************************************************************8
template < class KEY, class VALUE, class ValueHash, class ValueEqual >
int MinHeap< KEY, VALUE, ValueHash, ValueEqual>::parent(int pos) const { // Return position for parent
  assert(pos > 0);                // Posmust have a parent
  return (pos-1)/2;
}
//*******************************************************************

template < class KEY, class VALUE, class ValueHash, class ValueEqual >
void MinHeap< KEY, VALUE, ValueHash, ValueEqual>::insert(const ELEM<KEY, VALUE> e) { // Insert value into heap
  assert(n < size);
  int curr = n++;
  (*Heap)[curr] = e;// Start at end of heap
  (*refmap)[e.value]=curr;
  // Now sift up until curr's parent < curr
  while ((curr!=0) && (key((*Heap)[curr]) < key((*Heap)[parent(curr)]))) {
    Swap(curr, parent(curr));

    curr = parent(curr);
  }
}
//********************************************************************

template < class KEY, class VALUE, class ValueHash, class ValueEqual >
ELEM<KEY, VALUE> MinHeap< KEY, VALUE, ValueHash, ValueEqual>::removeMin() {          // Remove minimum value
  assert(n > 0);
  Swap(0, --n); // Swap maximum with last value
  if (n != 0)      // Not on last element
    siftDown(0);   // Put new heap root val in correct place
  (*refmap)[(*Heap)[n].value]=-1;
  return (*Heap)[n];
}
//*******************************************************************

// Remove value at specified position
template < class KEY, class VALUE, class ValueHash, class ValueEqual >
ELEM<KEY, VALUE> MinHeap< KEY, VALUE, ValueHash, ValueEqual>::remove(int pos) {
  assert((pos >= 0) && (pos < n));
  Swap(pos, --n); // Swap with last value
  if (n > 0 && pos < n)        // Not on last element
    siftDown(pos);   // Put new heap root val in correct place
  (*refmap)[(*Heap)[n].value]=-1;
  return (*Heap)[n];
}
//********************************************************************

template < class KEY, class VALUE, class ValueHash, class ValueEqual >
void MinHeap< KEY, VALUE, ValueHash, ValueEqual>::buildHeap()           // Heapify contents of Heap
{ for (int i=n/2-1; i>=0; i--) siftDown(i); }

template < class KEY, class VALUE, class ValueHash, class ValueEqual >
void MinHeap< KEY, VALUE, ValueHash, ValueEqual>::siftDown(int pos) { // Put element in its correct place
  assert((pos >= 0) && (pos < n));
  while (!isLeaf(pos)) {
    int j = leftChild(pos);
    if ((j<(n-1)) && (key((*Heap)[j]) > key((*Heap)[j+1])))
      j++; // j is now index of child with greater value
   // cout<<"before calling <=: "<<pos<<" and "<<j<<endl;
   // cout<<n<<endl;
    if (key((*Heap)[pos]) <= key((*Heap)[j]))
    	return; // Done
    Swap(pos, j);
    pos = j;  // Move down
  }
}
//*****************************************************************
template < class KEY, class VALUE, class ValueHash, class ValueEqual >
ELEM<KEY, VALUE> MinHeap< KEY, VALUE, ValueHash, ValueEqual>::replaceMin(ELEM<KEY, VALUE> x) //replace the min value with x
{
	ELEM<KEY, VALUE> temp=(*Heap)[0];
	(*Heap)[0]=x;
	siftDown(0);
	return temp;
}
#endif
