#include "summarizer.h"
#ifdef LARGE_DATASET 
struct Prefetch prefetch[MEM_PREFETCH];
#endif

Summarizer::~Summarizer()
{
	for(unsigned int i=0; i<colorArr.size(); i++)
	{
		delete colorArr[i];
		colorArr[i]=NULL;
	}
}

void Summarizer::groupByAttribute()
{
	UINT* nodeColorMap;
	intializeColorArray(nodeColorMap);
//	debug(33, "initialized colorarray\n");
	initializePaticipates(nodeColorMap);
//	debug(33, "initialized participates\n");
	delete [] nodeColorMap;
}
void Summarizer::intializeColorArray(UINT*& nodeColorMap)
{
	vector<int> nodes;
	vector<string> values;
	db->groupNodesByAttr(tablePrefix+"_Node", Color::gid, Color::nodeAttr, nodes, values);

	if (nodes.size() == 0 && values.size() == 0)
	{
		cerr<<"Error: wrong attribute name or graph id"<<endl;
		exit(-1);
	}

	colorArr.clear();
	
	string preValue="";
	
	int colorid=-1;
	ColorArray::iterator pcurrColor;

	nodeColorMap=new UINT[nodes.size()];
	for(unsigned int i=0; i<nodes.size(); i++)
	{
		if(preValue!=values[i])
		{
			colorid++;
			preValue=values[i];
			pcurrColor=(colorArr.insert(ColorArray::value_type(colorid, new Color(colorid, colorid) ) ) ).first;
			Color::attrValues.push_back(values[i]);
		}
		pcurrColor->second->nodeMap.insert(NodeMap::value_type(nodes[i], new NodeBitArray(nodes[i])));
		nodeColorMap[nodes[i]]=colorid;
	}
}

void Summarizer::initializePaticipates(UINT* nodeColorMap)
{
	vector<int> graphs;
	vector<int> node1s;
	vector<int> node2s;
	vector<Attributes*> vattrs;
	char buff[100];
	sprintf(buff, "where graphID=%d and attrname=\'type\' and attrvalue=\'%s\'", Color::gid, Color::edgeType.c_str());
	string condition=string(buff);
	
	db->selectEdges(tablePrefix+"_Edge", condition, graphs, node1s, node2s, vattrs);

	if (node1s.size() == 0 && node2s.size() == 0 && graphs.size() == 0 )
        {
		cerr<<"Error: wrong edgetype or graph id"<<endl;
		exit(-1);
	}

	string directed;
	db->getEdgeAttr(tablePrefix+"_Edge", Color::gid, node1s[0], node2s[0], "directed", directed);  // if the graph is directed or not

	if (directed == "true")
		directedGraph = true;
	else
		directedGraph = false;

	UINT colorid1, colorid2;
	UINT v1, v2;
	
	for(int i=0; i<graphs.size(); i++)
	{
		v1=node1s[i];
		v2=node2s[i];

		colorid1=nodeColorMap[v1];
		colorid2=nodeColorMap[v2];
		//modify the color array
		colorArr[colorid1]->nodeMap[v1]->setBitOn(colorid2);
		if (!directedGraph)
		colorArr[colorid2]->nodeMap[v2]->setBitOn(colorid1);

		#ifdef LARGE_DATASET               // if use LARGE_DATASET, then prefetch the node's neighbors into an array
		prefetch[v1].neighbor.push_back(v2);
		if (!directedGraph)
		prefetch[v2].neighbor.push_back(v1);
		#endif
	}
	
	//update the frequency counts of each colorArr
	ColorArray::iterator it;
	for(it=colorArr.begin(); it!=colorArr.end(); it++)
	{
		it->second->updateParticipates();
	}
	//clean the attribute pointers
	for(unsigned int i=0; i<vattrs.size(); i++)
	{
		delete vattrs[i];
	}
}
	
//after this function, nodes is sorted, and buckets give the information of the buckets 
void Summarizer::radix(UINT oc, vector<UINT>& nodes, vector<UINT>& digits, vector<Bucket>& buckets)
{
	//initialization
	queue<Bucket> q;
	Bucket b;
	b.start=0;
	b.count=(int)nodes.size();
	b.nextDigit=0;
	q.push(b);

	while(!q.empty())
	{
		Bucket cb=q.front();
		q.pop();
		
		if(cb.nextDigit>=digits.size())
		{
			if(cb.count>0)
				buckets.push_back(cb);
			continue;
		}
		
		vector<UINT> A, B;
		for(unsigned int i=cb.start; i<cb.start+cb.count; i++)
		{
			if( colorArr[oc]->nodeMap[nodes[i]]->checkBit(digits[cb.nextDigit]) )
				B.push_back(nodes[i]);
			else
				A.push_back(nodes[i]);
		}
		
		if(!A.empty())
		{
			Bucket ab;
			ab.start=cb.start;
			ab.count=A.size();
			ab.nextDigit=cb.nextDigit+1;
			q.push(ab);
			
			for(unsigned int i=0; i<ab.count; i++)
				nodes[i+ab.start]=A[i];
		}
		if(!B.empty())
		{
			Bucket bb;
			bb.start=cb.start+A.size();
			bb.count=B.size();
			bb.nextDigit=cb.nextDigit+1;
			q.push(bb);
			
			for(unsigned int i=0; i<bb.count; i++)
				nodes[i+bb.start]=B[i];
		}
	}
}

void Summarizer::getBuckets(UINT oc, vector<UINT>& nodes, vector<UINT>& bases, vector<Bucket>& buckets, bool limit=false, int k=-1)
{
	buckets.clear();
	NodeMap::iterator p;
	nodes.clear();
	for(p=colorArr[oc]->nodeMap.begin(); p!=colorArr[oc]->nodeMap.end(); p++)
		nodes.push_back(p->first);
	//sort the nodemaps using radix sort
	radix(oc, nodes, bases, buckets);
	
	if(!limit)
		return;
	
//	int groupsNeed=k-colorArr.size();
//	if((int)buckets.size()-1>groupsNeed)
//		reduceBuckets(oc, nodes, buckets, groupsNeed);
	
}

void Summarizer::reduceBuckets(UINT oc, vector<UINT>& nodes, vector<Bucket>& buckets, int groupsNeed)
{

	hash_map<int, hash_set<int> > buckgroups;
	vector<SubColor*> buckcolors;
	
	//initialize heap;
	vector< ELEM<GroupDistance, ColorEdge> >* h=new vector< ELEM<GroupDistance, ColorEdge> >();
	
	for(unsigned int i=0; i<buckets.size(); i++)
	{
		buckgroups.insert(hash_map<int, hash_set<int> >::value_type(i, hash_set<int>()));
		buckgroups[i].insert(i);
		
		SubColor* scolor=new SubColor;
		scolor->size=buckets[i].count;
		colorArr[oc]->nodeMap[nodes[buckets[i].start]]->addBits(scolor->participates);
		hash_map<UINT, int>::iterator p;
		for(p=scolor->participates.begin(); p!=scolor->participates.end(); p++)
		{
			UINT color=p->first;
			int pt=p->second;
			if(pt==0) continue;
			scolor->setParticipates(color, buckets[i].count);
		}
		buckcolors.push_back(scolor);	
	}
	
	for(unsigned int i=0; i<buckets.size(); i++)
		for(unsigned int j=i+1; j<buckets.size(); j++)
		{
			ELEM<GroupDistance, ColorEdge> elem;
			
			elem.value.colorid1=i;
			elem.value.colorid2=j;
			
			elem.key=calcGroupDist(buckcolors[elem.value.colorid1], buckcolors[elem.value.colorid2]);
			h->push_back(elem);
		}
	
	MinHeap<GroupDistance, ColorEdge, colorEdgehash, eqColorEdge>* heap=new MinHeap<GroupDistance, ColorEdge, colorEdgehash, eqColorEdge>(h, (int)h->size(), (int)h->size());
//	cout<<"after initialization, the heap is:\n";
//	heap->printHeap();
	hash_map<int, hash_set<int> >::iterator it;
	while((int)buckgroups.size()-1>groupsNeed)
	{
		ELEM<GroupDistance, ColorEdge> e=heap->removeMin();
		
		//merge the participates
		buckcolors[e.value.colorid1]->mergeIn(buckcolors[e.value.colorid2]);
		buckcolors[e.value.colorid2]->size=0;
		
		//merge the two groups
		buckgroups[e.value.colorid1].insert(buckgroups[e.value.colorid2].begin(), buckgroups[e.value.colorid2].end());
		buckgroups.erase(e.value.colorid2);
		
		//update the heap
		for(it=buckgroups.begin(); it!=buckgroups.end(); it++)
		{
			if(it->first==e.value.colorid1)
				continue;
			//change the some entries
			ELEM<GroupDistance, ColorEdge> te;
			if(it->first<e.value.colorid1)
			{
				te.value.colorid1=it->first;
				te.value.colorid2=e.value.colorid1;
			}else
			{
				te.value.colorid2=it->first;
				te.value.colorid1=e.value.colorid1;
			}
			te.key=calcGroupDist(buckcolors[te.value.colorid1], buckcolors[te.value.colorid2]);

			heap->changeKey(te);
			
			//delete some entries
			if(it->first<e.value.colorid2)
			{
				te.value.colorid1=it->first;
				te.value.colorid2=e.value.colorid2;
			}else
			{
				te.value.colorid2=it->first;
				te.value.colorid1=e.value.colorid2;
			}
			heap->removeKey(te.value);
		}
	}
	
	//change the buckets based on the merge results
	vector<UINT> nodescopy(nodes);
	vector<Bucket> bucketscopy(buckets);
	buckets.clear();
	int start=0;
	for(it=buckgroups.begin(); it!=buckgroups.end(); it++)
	{
		Bucket buck;
		buck.start=start;
		hash_set<int>::iterator q;
		buck.count=0;
		for(q=it->second.begin(); q!=it->second.end(); q++)
		{
			//update nodes
			int bstart=bucketscopy[*q].start;
			int bcount=bucketscopy[*q].count;
			for(int i=0; i<bcount; i++)
				nodes[start+i]=nodescopy[bstart+i];
			buck.count+=bcount;
			start+=bcount;
		}
		//update buckets
		buckets.push_back(buck);
	}
	
	for(unsigned int i=0; i<buckcolors.size(); i++)
		delete buckcolors[i];
	delete h;
	delete heap;
}

//split an existing group into several groups based on the edges connecting
//to the group in the bases
void Summarizer::splitColorUpdate( UINT oc, vector<UINT>& bases, hash_set<UINT>& colorChanged, bool limit, int k)
{
	vector<Bucket> buckets;
	vector<UINT> nodes;
	getBuckets(oc, nodes, bases, buckets, limit, k);
	debug(33, "get buckets\n");
	colorChanged.clear();
	//split color oc
	hash_map<UINT, int> ocpartcp(colorArr[oc]->participates);
	
	hash_map<UINT, UINT> ncmap;
	unsigned int b;
	for(b=0; b<buckets.size()-1; b++)
	{
		assert(buckets[b].count>0);
		//cout<<"bucket.count = "<<buckets[b].count<<endl;
		int newindx=Color::maxColorid+1;
		Color* pncolor=new Color(colorArr[oc]->initialColor, newindx);
		colorArr.insert(ColorArray::value_type(newindx, pncolor));
		for(unsigned int i=buckets[b].start; i<buckets[b].start+buckets[b].count; i++)
		{
			ncmap[nodes[i]]=newindx;
			pncolor->nodeMap.insert(NodeMap::value_type(nodes[i], colorArr[oc]->nodeMap[nodes[i]]));
		}
		//pncolor->updateParticipates();
		colorChanged.insert(newindx);
	}
	
	debug(33, "newed color\n");
	//the last new color uses the existing color
	NodeMap tempmap;
	for(unsigned int i=buckets[b].start; i<buckets[b].start+buckets[b].count; i++)
	{
		ncmap[nodes[i]]=oc;
		tempmap.insert(NodeMap::value_type(nodes[i], colorArr[oc]->nodeMap[nodes[i]]));
		
	}
	colorArr[oc]->nodeMap.clear();
	colorArr[oc]->nodeMap=tempmap;
	//colorArr[oc]->updateParticipates();
	colorChanged.insert(oc);
	
	debug(33, "newed color2\n");
	
	//update the new colors;
	hash_set<UINT>::iterator q;
	for(q=colorChanged.begin(); q!=colorChanged.end(); q++)
	{
		updateAfterSplit(*q, oc, ncmap);
	}
	
	debug(33, "update new colors\n");
	
	//update neighboring colors
	hash_map<UINT, int>::iterator it;
	for(it=ocpartcp.begin(); it!=ocpartcp.end(); it++)
	{
		if(it->first!=oc && it->second>0)
		{
			updateAfterSplit(it->first, oc, ncmap);
			colorChanged.insert(it->first);
		}
	}
	debug(33, "update neighboring colors\n");
}

void Summarizer::updateAfterSplit( UINT tc, UINT oc, hash_map<UINT, UINT>& ncmap)
{	
	NodeMap::iterator p;
	for(p=colorArr[tc]->nodeMap.begin(); p!=colorArr[tc]->nodeMap.end(); p++)
	{
		if(!(p->second->checkBit(oc)) )
			continue;
		
		//reset the bits for the old color
		p->second->setBitOff(oc);

		#ifdef LARGE_DATASET 			//use the perfetched information (e.g. neighbors) in the array (fast)
		for (unsigned int i=0; i<prefetch[p->first].neighbor.size(); i++)
		{
			hash_map<UINT, UINT>::iterator ppos=ncmap.find(prefetch[p->first].neighbor[i]);
			if(ppos!=ncmap.end())
				p->second->setBitOn(ppos->second);
		}
		#else					//or use query language to get the neighborhood information (slow)
		vector<int> neighbors;
		if(neighborplan)
		{
			db->selectNeighbors(tablePrefix+"_edge", Color::gid, p->first, "", neighbors, tablePrefix+"_neighborplan");
		}else
		{
			char buff[100];
			sprintf(buff, "attrname=\'type\' and attrvalue=\'%s\'",Color::edgeType.c_str());
			string condition=string(buff);
		//	debug(33, "before selecting neighbors\n");
			
			db->selectNeighbors(tablePrefix+"_edge", Color::gid, p->first, condition, neighbors, "");
		}
		//	debug(33, "selected neighbors\n");
		for (unsigned int i=0; i<neighbors.size(); i++)
		{
			hash_map<UINT, UINT>::iterator ppos=ncmap.find(neighbors[i]);
			if(ppos!=ncmap.end())
				p->second->setBitOn(ppos->second);
		}
		#endif
		
	}
	colorArr[tc]->updateParticipates();
}

//split an existing color into two based on the color base, 
//frequencies are partially updated
void Summarizer::splitColor(UINT oc, UINT nc, int base)
{	
	NodeMap::iterator p;
	NodeMap tempmap;
	
	colorArr[oc]->participates.clear();
	colorArr[nc]->participates.clear();
	
	for(p=colorArr[oc]->nodeMap.begin(); p!=colorArr[oc]->nodeMap.end(); p++)
	{
		if( p->second->checkBit(base) )
		{
			p->second->addBits(colorArr[oc]->participates);
			tempmap.insert(*p);
		}else
		{
			p->second->addBits(colorArr[nc]->participates);
			colorArr[nc]->nodeMap.insert(*p);
		}
	}
	colorArr[oc]->nodeMap.clear();
	colorArr[oc]->nodeMap=tempmap;
	
	//in case if the new color may interact with
	// the old color
	colorArr[oc]->setParticipates(nc, colorArr[oc]->getParticipates(oc));
	colorArr[nc]->setParticipates(nc, colorArr[nc]->getParticipates(oc));
	
}

//split, base, maxdelta are return value
void Summarizer::SNAP()
{
	//O(m'E) m' may be greater than m, plot m' vs. K and N
	int c1;
	vector<UINT> bases;
	while(colorToSplit(c1, bases))
	{
		hash_set<UINT> colorChanged;
		splitColorUpdate(c1, bases, colorChanged, false, 0);
		//printSummary();
	}
}


int Summarizer::getPanelty2(Color* color)
{
	int ret=0;
	int size1=color->nodeMap.size();
	UINT colorid1=color->colorid;
	hash_map<UINT, int>::iterator q;
	for(q=color->participates.begin(); q!=color->participates.end(); q++)
	{
		if(q->second<=0)
			continue;
		int part1=q->second;
		int size2=colorArr[q->first]->nodeMap.size();
		int part2=colorArr[q->first]->getParticipates(colorid1);
		if(part1==size1)
			continue;
		else if(part1+part2 < size1+size2-part1-part2)
			ret+=part1;
		else
			ret+=size1-part1;
	}
	return -ret;
}

int Summarizer::getPanelty(Color* color)
{
	int ret=0;
	int size1=color->nodeMap.size();
	UINT colorid1=color->colorid;
	hash_map<UINT, int>::iterator q;
	if (!directedGraph)
	{
		for(q=color->participates.begin(); q!=color->participates.end(); q++)
		{
			if(q->second<=0)
				continue;
			int part1=q->second;
			int size2=colorArr[q->first]->nodeMap.size();
			int part2=colorArr[q->first]->getParticipates(colorid1);

			if(part1==size1)
				continue;
			else if(part1+part2 < size1+size2-part1-part2)    // if p_{i,j} <= 0.5
				ret=max(ret, part1);			  // \delta =  p_{i,j}
			else
				ret=max(ret, size1-part1);		  // otherwise, \delta = |g_{i}| - |p_{i,j}|
		}
	}
	else
	{
		
		for(q=color->participates.begin(); q!=color->participates.end(); q++)
		{
			if(q->second<=0)
				continue;
			int part1=q->second;

			if(part1==size1)
				continue;

			else if(part1 < size1-part1)
				ret=max(ret, part1);
			else
				ret=max(ret, size1-part1);
		}
	}
	return -ret;
}

void Summarizer::updateHeap(MinHeap<int, UINT, hash<UINT>, equal_to<UINT> >* heap, hash_set<UINT>& colorChanged)
{
	hash_set<UINT>::iterator it;
	for(it=colorChanged.begin(); it!=colorChanged.end(); it++)
	{
		ELEM<int, UINT> tempe;
		tempe.value=*it;
		tempe.key=getPanelty(colorArr[*it]);
		//heap->printHeap();
		//heap->printRefMap();

		heap->removeKey(tempe.value);
		if(tempe.key==0)
			continue;

		heap->insert(tempe);
	}
}

void Summarizer::delectBases(UINT oc, vector<UINT>& bases)
{
	bases.clear();
	Color* color=colorArr[oc];
	int size1=color->nodeMap.size();
	hash_map<UINT, int>::iterator q;
	float mindelta=0.5;
	int mincolor=-1;
	for(q=color->participates.begin(); q!=color->participates.end(); q++)
	{
		if(q->second<=0)
			continue;
		int part1=q->second;

		
		float freq1 = (float)part1/(float)size1;
		float delta1= fabs(freq1-0.5);
		
		if(delta1<DELTA)// && part1>=part2)
		{
			bases.push_back(q->first);
		}
		
		if(delta1<mindelta)
		{
			mindelta=delta1;
			mincolor=q->first;
		}

	}
	
	if(bases.empty())
		bases.push_back(mincolor);
}

//find oc's neighboring group g_{j} that achieves the largest \delta value
//store g_{j} into bases
void Summarizer::delectOneBase(UINT oc, vector<UINT>& bases)
{
	bases.clear();
	Color* color=colorArr[oc];
	int size1=color->nodeMap.size();
	hash_map<UINT, int>::iterator q;
	float mindelta=0.5;
	int mincolor=-1;
	//cout<<"the color inspecting: "<<oc<<endl;
	for(q=color->participates.begin(); q!=color->participates.end(); q++)
	{
		if(q->second<=0)
			continue;
		int part1=q->second;
		
		float freq1 = (float)part1/(float)size1;
		float delta1= fabs(freq1-0.5);
			
		if(delta1<mindelta)
		{
			mindelta=delta1;
			mincolor=q->first;
		}

	}
	
	bases.push_back(mincolor);
}
/*
void Summarizer::printSummary()
{
        ColorArray::iterator it;
        for(it=colorArr.begin(); it!=colorArr.end(); it++)
        {
        	cout<<"Color"<<colorArr[i].colorid<<"\tIniticalColor:"
        	<<colorArr[i].initialColor<<"\tSize:"<<colorArr[i].nodeMap.size();
        }
}
*/
void Summarizer::printSummary()
{
        cout<<"-----------------------------\n";
        cout<<"#Groups: "<<colorArr.size()<<endl;
        //print Nodes first
        ColorArray::iterator it;
        for(it=colorArr.begin(); it!=colorArr.end(); it++)
                it->second->print();
                                                                                
        #ifdef PRINT_RELATIONSHIP
                                                                                
        //cout<<"\n###############################################\n";
        cout<<"Group Relationships: (color1, color2, #edges, freq, part1, part2): \n";
        ColorArray::iterator p;
        hash_map<UINT, int>::iterator q;
        int distance=0;
        int numGroupEdge=0;
        for(p=colorArr.begin(); p!=colorArr.end(); p++)
        {
                UINT color1=p->first;
                int total1=p->second->getSize();
                for(q=p->second->participates.begin(); q!=p->second->participates.end(); q++)
                {
                                                                                
                        UINT color2=q->first;
                        if(q->second==0 || color1>color2)
                                continue;
                                                                                
                        int part1=q->second;
                        int part2=colorArr[color2]->getParticipates(color1);
                        assert(part2>0);
                        int total2=colorArr[color2]->getSize();
                                                                                
                        int part=part1+part2;
                        int total=total1+total2;
                        distance+=min(part, total-part);
                                                                                
                        numGroupEdge++;
                }
        }
        cout<<"#relationships: "<<numGroupEdge<<endl;
        cout<<"** DELTA MEASURE = "<<distance<<" **"<<endl;
        cout<<"** NORMALIZED DELTA MEASURE = "<<(float)distance/(float)(colorArr.size())<<endl;
        #endif
}

void Summarizer::kSNAP(int k)
{
	//initialize the heap
	int maxHeapSize=k+100;
	vector< ELEM<int, UINT> >* h=new vector< ELEM<int, UINT> >(maxHeapSize);
	
	ColorArray::iterator p;
	unsigned int i=0;
	for(p=colorArr.begin(); p!=colorArr.end(); p++)
	{
		ELEM<int, UINT> e;
		
		e.value=p->first;
		e.key=getPanelty(p->second);
		if(e.key==0)
			continue;
		(*h)[i]=e;
		i++;
	}
	
	MinHeap<int, UINT, hash<UINT>, equal_to<UINT> >* heap=new MinHeap<int, UINT, hash<UINT>, equal_to<UINT> >(h, (int)i, maxHeapSize);
	
	while((int)colorArr.size()<k && !heap->isEmpty())
	{

		ELEM<int, UINT> e=heap->removeMin();             // choose the largest \delta value in the heap
		UINT colorToSplit=e.value;			 // select the group g_{i} to be split
		//cout<<"e.key = "<<e.key<<endl;
		//cout<<"colorToSplit: "<<colorToSplit<<endl;
		if(e.key==0)
			break;

		vector<UINT> bases;
		delectOneBase(colorToSplit, bases);		// find g_{i}'s neighboring group g_{j} that achieves the largest \delta value
								// save g_{j} into bases

		//for(unsigned int i=0; i<bases.size(); i++)
		//	cout<<"reference:"<<bases[i]<<"\t";
		//cout<<endl;
		//printSummary();
		
		hash_set<UINT> colorChanged;
		splitColorUpdate(colorToSplit, bases, colorChanged, false, 0);    //split group g_{i} based on whether nodes in g_{i} connect to nodes in the neighboring group g_{j}
//		debug(33, "split color update\n");
		updateHeap(heap, colorChanged);			// update the heap and data structure
//		debug(33, "update heap\n");
	}
	printSummary();
	delete h;
	delete heap;
}

bool Summarizer::colorToSplit(int& split, vector<UINT>& bases)
{
	bases.clear();
	ColorArray::iterator p;
	hash_map<UINT, int>::iterator q;
	for(p=colorArr.begin(); p!=colorArr.end(); p++)
	{
		split=p->first;
		for(q=p->second->participates.begin(); q!=p->second->participates.end(); q++)
		{
			if(q->second>0 && q->second<(int)p->second->nodeMap.size())
			{
				bases.push_back(q->first);
			}	
		}
		if(!bases.empty())
			return true;
	}
	return false;
}

void Summarizer::validate()
{
	//build a hash_map to map nodeid to colorid
	hash_map<UINT, UINT> nodeColorRef;
	ColorArray::iterator it;
	for(it=colorArr.begin(); it!=colorArr.end(); it++)
	{
		NodeMap::iterator p;
		for(p=it->second->nodeMap.begin(); p!=it->second->nodeMap.end(); p++)
			nodeColorRef.insert(hash_map<UINT, UINT>::value_type(p->first, it->first));
	}
	
	for(it=colorArr.begin(); it!=colorArr.end(); it++)
		validateColor(nodeColorRef, it->first);
}
void Summarizer::validateColor(hash_map<UINT, UINT>& ref, UINT colorid)
{
	NodeBitArray bitArr(0);
	NodeMap::iterator p;
	char buff[100];
	sprintf(buff, "attrname=\'type\' and attrvalue=%s;", Color::edgeType.c_str());
	string condition=string(buff);
	for(p=colorArr[colorid]->nodeMap.begin(); p!=colorArr[colorid]->nodeMap.end(); p++)
	{	
		//first check the node neighbor color bitmap
		vector<int> neighbors;
		db->selectNeighbors(tablePrefix+"_edge", Color::gid, p->first, condition, neighbors, "");
		
		for (unsigned int i=0; i<neighbors.size(); i++)
		{
			bitArr.setBitOn(ref[neighbors[i]]);
		}
		if(!p->second->equal(bitArr))
		{
			cout<<"validation failed for color "<<colorid<<endl;
			cout<<"at node "<<p->first<<endl;
			exit(-1);
		}
		bitArr.resetAll();
	}
}
