#ifndef SUMMERIZER_H
#define SUMMERIZER_H

#include "color.h"
#include <algorithm>
#include "datastruct.h"
#include "../util/minheap.h"
#include <hash_set.h>
#include <queue>
#include <iomanip>
#include "../util/mytimer.h"
#include "../db/db.h"
#include <math.h>

//#define VALIDATE
#define PRINT_RELATIONSHIP
#define DELTA 0.3

/**
  * If the graph to be summarized is very large, we suggest keeping some auxiliary information in memory to speed up the process of summarization. To enable this feature, compile the   
  * program with LARGE_DATASET switched ON, and the value of MEM_PREFETCH should be set to the number of nodes in the 
  * original input graph. If there is a segmentation fault when you run the code, it means the machine memory is not big enough to hold the auxiliary information. In this released package, 
  * we simply provide the most straightforward method to store the auxiliary information (e.g. use a two-dimensional array), and there must be some more efficient methods that use less 
  * memory to do the same jobs.
*/

#define LARGE_DATASET 

#define MEM_PREFETCH 20000


struct NodeReference
{
	UINT nodeid;
	UINT colorid;
};

struct Prefetch{
	vector<int> neighbor;
};

struct ColorBackUp
{
	int initialColor;
	vector<int> nodes;
	ColorBackUp(int initcolor)
	{
		initialColor=initcolor;
	}
};

struct Bucket
{
	unsigned int start;
	unsigned int count;
	unsigned int nextDigit;
};


class Summarizer
{
public:

	MicroTimer timer;
	double time;
	~Summarizer();
	
	Summarizer(DB* pdb)
	{
		timer.reset();
		time=0;
		db=pdb;
		neighborplan=false;
	}
	
	void setGraph(string prefix, int g, string attrname, string edgetype)
	{
		tablePrefix=prefix;
		Color::gid=g;
		Color::edgeType=edgetype;
		Color::nodeAttr=attrname;
		Color::maxColorid=-1;
		groupByAttribute();
		
		char qrystr[500];
		neighborplan=true;
		sprintf(qrystr, "prepare %s_neighborplan (int4, int4) as select nodeID from "
				"( (select node2ID as nodeID from %s_edge where attrname=\'type\' "
				"and attrvalue=\'%s\' and graphID=$1 and node1ID=$2) union "
				"(select node1ID as nodeID from %s_edge where attrname=\'type\' "
				"and attrvalue=\'%s\' and graphID=$1 and node2ID=$2) ) t;", 
				tablePrefix.c_str(), tablePrefix.c_str(), edgetype.c_str(), 
				tablePrefix.c_str(), edgetype.c_str());
		db->preparePlan(string(qrystr));
	}
	
	//after this function, nodes is sorted, and buckets give the information of the buckets 
	void radix(UINT oc, vector<UINT>& nodes, vector<UINT>& digits, vector<Bucket>& buckets);
	
	void getBuckets(UINT oc, vector<UINT>& nodes, vector<UINT>& bases, vector<Bucket>& buckets, bool limit, int k);
	
	void reduceBuckets(UINT oc, vector<UINT>& nodes, vector<Bucket>& buckets, int groupsNeed);
	//split an existing color into several colors based on the edges connecting
	//to the colors in the bases
	void splitColorUpdate( UINT oc, vector<UINT>& bases, hash_set<UINT>& colorChanged, bool limit, int k);
	void updateAfterSplit( UINT tc, UINT oc, hash_map<UINT, UINT>& ncmap);
	
	//split an existing color into two based on the color base, 
	//frequencies are partially updated
	void splitColor(UINT oc, UINT nc, int base);
	
	//split, base, maxdelta are return value
	void SNAP();
	int getPanelty2(Color* color);
	int getPanelty(Color* color);
	
	void updateHeap(MinHeap<int, UINT, hash<UINT>, equal_to<UINT> >* heap, hash_set<UINT>& colorChanged);
	
	void delectBases(UINT oc, vector<UINT>& bases);
	void delectOneBase(UINT oc, vector<UINT>& bases);
	
	void kSNAP(int k);
	bool colorToSplit(int& split, vector<UINT>& bases);
	
	void validate();
	void validateColor(hash_map<UINT, UINT>& ref, UINT colorid);
	void groupByAttribute();
	void intializeColorArray(UINT*& nodeColorMap);
	void initializePaticipates(UINT* nodeColorMap);
	void printSummary();
	
	DB* db;
	ColorArray colorArr;
	bool directedGraph;   // indicate if the graph is directed or not. There are some differences when dealing with directed and undirected graph.

private:
	string tablePrefix;
	bool neighborplan;
	
};

#endif
