/******************************************************************************
 * Delaration for some utility functions for manipulating graphs
 * Author: Yuanyuan Tian (ytian@umich.edu)
 * ***************************************************************************/

#ifndef GRAPHUTIL_H

#define GRAPHUTIL_H

#include "../graph/hashgraph.h"
#include <set>
#include "../orthology/ortholog.h"
#include <hash_set.h>
#include <iostream>

#define INFIN INT_MAX/10

struct Neighborhood
{
	int v;
	int degree;
	int nbConn;
	bool nbArray[96];
};


//Get the adjacency matrix of the graph
void getMatrices(
		HashGraph* G,	//input graph
		int** D			//output matrix (D is allocated outside)
		);

//Compute all pairwise shortest distance using Floyd's algorithm
//NOTE: The distance does not consider the weight of edge.
void floyd_Min_Dist(
		HashGraph* G,	//input graph 
		int** D			//output distance matrix
						//(D is allocated outside)
		);

//Find the nodes on the shortest path between v1 and v2
//NOTE: The distance does not consider the weight of edge.
bool getBridge(
		HashGraph* G,	//input graph 
		int v1, 		//one end node
		int v2, 		//the other end node
		int len, 		//length of the shortest path (input)
		set<int>& vset	//returned set of ndoes
		);

//Compute the shortest paths from s to other nodes in the graph
//Dijkstra's shortest paths algorithm w/ priority queue
//NOTE: The distance does not consider the weight of edge.
void Dijkstra(
		HashGraph* G,		//input graph 
		int* D, 		//returned distance matrix
		int s			//source node#include "common.h"
		);

//Compute the pairwise shortest distance among the set nodes in V
void getDistance(
		HashGraph* G, 	//input graph
		int** D,		//the returned distance matrix 
		hash_set<int>& V //The input set of nodes
		);


//return the neighborhood of a node
void getNeighborhood(
		HashGraph* G, 	//input graph
		OrthologInfoList& orthinfolist,	//the orthology info for the ndoes in G 
		int v, 	//the input node
		Neighborhood& nb	//the output neighborhood
		);

void getAllNeighborhoods(
		HashGraph* G, 	//input graph
		OrthologInfoList& orthinfolist,	//the orthology info for the ndoes in G 
		vector<Neighborhood>& nbs	//the output neighborhoods
		);

//return the top k important nodes based on degree centrality
void topKImpNodes(
		HashGraph* G, 	//input graph
		OrthologInfoList& orthinfolist,  //the orthology info for the ndoes in G 
		int& k,	//num of important nodes
		vector<int>& nodes	//returned important nodes
		);
#endif
