/******************************************************************************
 * Delaration for types and functions related to a distance matrix
 * Author: Yuanyuan Tian (ytian@umich.edu)
 * ***************************************************************************/

#ifndef MATRIX_H
#define MATRIX_H

#include <vector>
#include <string>
#include <set>
#include <hash_map.h>
#include "../orthology/ortholog.h"
#include <assert.h>
using namespace std;
//**********************************************
// The Subset of nodes
//**********************************************

typedef vector<int> NodeSet;//each element in this set is a vertex id

//less than operation for the NodeSet
struct ltsubset
{
	bool operator()(const NodeSet s1, const NodeSet s2) const
	{
		if(s1.size()!=s2.size())
			return ( s1.size()<s2.size() );
			
		for(unsigned int i=0; i<s1.size(); i++)
		{
			if(s1[i]!=s2[i])
				return (s1[i]<s2[i]);
		}
		return false;
	}
};

//translate a string of vetexset into a vertex set
//vset is the return value
inline void getVertexSet(string vstr, NodeSet& vset)
{
	int start=0;
	int temp=0;
	while(start<(int)vstr.length())
	{
		temp=vstr.find("*", start);
		assert(temp>=start);
		string str=vstr.substr(start, temp-start);
		vset.push_back(atoi((char*)str.c_str()));
		start=temp+1;
	}
}

//add a vertex id into the vertexset, the vertexset is ordered by the id
inline void addOneNode(NodeSet& vs, int node)
{
	NodeSet::iterator iter=vs.begin();
	for(; iter!=vs.end(); iter++)
	{
		if(node<(*iter))
			break;
	}
	vs.insert(iter, node);
}

//**********************************************
// lists of vertex subset
//**********************************************
typedef vector<NodeSet> NodeSetList;// a list of NodeSet, unordered.

//**********************************************
// distance matrix of  vertex subset
//**********************************************

//the distance matrix, right now it is just a array of the elements in the matrix
typedef vector<int> Matrix;

//given two summary get graph distance
//getDistance
// dist is the true return value, the distance between the two fragments,
//the function returns true if every pairwise difference is not greater than threshold.
bool getStructDist(Matrix& qm, Matrix& dbm, int threshold, int& dist);

//get the sum of the matrix values
int getMatrixSum(Matrix& matrix);

//return matrix and nodeset
void getMatrix(int** D, vector<NodeOrtholog>& vfset, Matrix& matrix, NodeSet& nset);

//**************************************************
//Axulary data structures used 
//**************************************************
struct Fragment
{
	NodeSet nset;// the node set
	Matrix matrix; //the summary associated
};

//**************************************************
typedef vector<Fragment> FragmentList;

//*************************************************

struct IndexSum
{
	unsigned int index;//the index in the Fragment list
	int matrixSum; //the sum of the matrix
};

//used to order IndexSum objects , order by (matrixSum, index) 
struct ltIS
{
	bool operator () (const IndexSum is1, const IndexSum is2)
	{
		if(is1.matrixSum!=is2.matrixSum)
			return is1.matrixSum<is2.matrixSum;
		else
			return is1.index<is2.index;
	}
	
};

typedef set<IndexSum, ltIS> IndexSumSet;

//**************************************************

//map the orthSeq code to the set of fragments with the same gSeq
typedef hash_map<uint64_t, IndexSumSet*, inthash> OrthSeqFragmentMap;

#endif

