/******************************************************************************
 * Definition for some utility functions
 * Author: Yuanyuan Tian (ytian@umich.edu)
 * ***************************************************************************/
#include "util.h"

unsigned int hashfunc(const string label)
{
    unsigned int hash = 5381;
    int c;
    for(unsigned int i=0; i<label.size(); i++)
    {
            c=label[i];
            hash=((hash << 5) + hash) + c;
    }
    return hash;
}

unsigned int hashfunc(unsigned int key)
{
	unsigned int c2=0x27d4eb2d; // a prime or an odd constant
	key = (key ^ 61) ^ (key >> 16);
	key = key + (key << 3);
	key = key ^ (key >> 4);
	key = key * c2;
	key = key ^ (key >> 15);
	return key;
}

unsigned int hashfunc(uint64_t key)
{
	uint64_t c2=0x27d4eb2d;
	key = (key ^ 61) ^ (key >> 16);
	key = key + (key << 3);
	key = key ^ (key >> 4);
	key = key * c2;
	key = key ^ (key >> 15);
	//uint32_t* p=&key;
	return (unsigned int) (key);
}

string trimStr(string str)
{
        int start, end;
        start=str.find_first_not_of(" \t\n\r");
        end=str.find_last_not_of(" \t\n\r");
        if(start<0)
                return "";
        else
                return str.substr(start, end-start+1);
}

string tolower(string str)
{
        for(unsigned int i=0; i<str.size(); i++)
                str[i]=tolower(str[i]);
        return str;
}

void replaceBadChar(string& str, string bad, char good)
{
	int i=0;
	//while( (i=str.find_first_of("\'\"", i))>=0 )
	while( (i=str.find_first_of(bad, i))>=0 )
		str[i]=good;
}

void replaceBadChar(string& str, char bad, string good)
{
	int i=0;
	//while( (i=str.find_first_of("\'\"", i))>=0 )
	while( (i=str.find_first_of(bad, i))>=0 )
	{
		str.replace(i, 1, good);
		i+=good.size();
	}
}

//helper function for FragmentIndex in SAGA
uint64_t getOrthSeq(const vector<int>& orthlogs)
{
	assert(orthlogs.size()<=4);
    uint64_t ret=0;
    for(unsigned int i=0; i<orthlogs.size(); i++)
    {
            ret=ret<<16;
            ret=ret|(uint64_t)orthlogs[i];
    }
    return ret;
}


void getDistSeqSum(const vector<uint32_t>& distMatrix, 
		uint32_t& summary, string& matrixStr)
{
	summary=0;
	char buff[50];
	matrixStr="{";
	for(unsigned int i=0; i<distMatrix.size(); i++)
    {
		summary+=distMatrix[i];
        sprintf(buff, "%u,", distMatrix[i]);
        matrixStr+=string(buff);
    }
	matrixStr[matrixStr.size()-1]='}';
}

void getNodeSeq(const vector<int>& nodes, string& nodeSeq)
{
	nodeSeq="{";
	char buff[50];
	for(unsigned int i=0; i<nodes.size(); i++)
    {
        sprintf(buff, "%d,", nodes[i]);
        nodeSeq+=string(buff);
    }
	nodeSeq[nodeSeq.size()-1]='}';
}

int difference(const vector<int>& vec1, const vector<int>& vec2)
{
	int ret=0;
	assert(vec1.size()==vec2.size());
	for(unsigned int i=0; i<vec1.size(); i++)
		ret+=abs(vec1[i]-vec2[i]);
	return ret;
}

void getBitString(const vector<bool>& bitarry, char str[])
{
    for(unsigned int i=0; i<96; i++)
    {
    	if(bitarry[i])
    		str[i]='1';
        else
            str[i]='0';
    }                                                                                                                                                     
}

void getBoolArray(vector<unsigned short> nbarray, bool* boolarray)
{
    int bitsize=sizeof(unsigned short)*8;
    for(unsigned int i=0; i<nbarray.size(); i++)
    {
    	unsigned short mask=1;
        mask<<=(bitsize-1);
        for(int j=0; j<bitsize; j++)
        {
                if(mask & nbarray[i])
                        boolarray[i*bitsize+j]=true;
                else
                        boolarray[i*bitsize+j]=false;
                mask>>=1;
        }
    }
}

bool isBitSet( unsigned short* array, int position)
{
        int unitsize=sizeof(unsigned short)*8;
        int start=position/unitsize;
        int offset=position%unitsize;
        unsigned short mask=1;
        mask<<=(unitsize-1-offset);
        return (mask & array[start]);
}

float calScore(int deg_q, int deg_db, int conn_q, int conn_db, int miss)
{
        float diff_deg= (deg_q<=deg_db)? 0: deg_q-deg_db;
        float diff_con= (conn_q<=conn_db)? 0: conn_q-conn_db;
        float score=0;
        if(deg_q>0)
                score= (miss > diff_deg)? (float)miss/(float)deg_q : diff_deg/(float)deg_q;
        if (score>1)    score=1;
        float f_con=0;
        if(conn_q>0 && diff_deg>0)
                f_con =  diff_con/(float)conn_q/diff_deg;
        else if(conn_q>0)
                f_con =  diff_con/(float)conn_q;
        score += f_con;
        return score;
}

bool readOneLine(FILE* f, string& str)
{
	char line[1501];
	str="";
	do
	{
		if(feof(f)) return false;
		fgets (line , 1501 , f);
		str+=string(line);
	}while(strlen(line)>=1500 && line[1499]!='\n');
	//str=str.substr(0, str.size()-1);//remove the '\n'
	if(str[str.size()-1]=='\n')
		str=str.substr(0, str.size()-1);
	return true;
}

int getOneSegment(string str, int start, char delim, string& seg)
{
	int end=str.find(delim, start);
	if(end!=string::npos)
	{
		seg=str.substr(start, end-start);
		end++;
	}
	else
		seg=str.substr(start);
	seg=trimStr(seg);
	return end;
}

void readItems(string str, char delim, vector<string>& items)
{
	int p=0;
	string item;
	do{
		p=getOneSegment(str, p, delim, item);
		items.push_back(item);
	}while(p!=string::npos);
}
