/******************************************************************************
 * Delaration for some utility functions
 * Author: Yuanyuan Tian (ytian@umich.edu)
 * ***************************************************************************/
#ifndef UTIL_H
#define UTIL_H
#include <string>
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "libpq-fe.h"
#include <vector>
#include <assert.h>
#include <string>
#include <sstream>
#include <math.h>
#include "../debug/debug.h"
using namespace std;

//change long integer from the network format to host format, or from host format to network format
#define ntohll(x) (((uint64_t)(ntohl((int)((x << 32) >> 32))) << 32) | (unsigned int)ntohl(((int)(x >> 32))))                                                                               
#define htonll(x) ntohll(x)

//hash functions
unsigned int hashfunc(const string label);
//unsigned int hashfunc(unsigned int key);
unsigned int hashfunc(uint64_t key);

//hash function used in hash_set or hash_map
struct strhash
{
	unsigned int operator() (const string key) const
	{
        return hashfunc(key);
	}
};

//hash function used in hash_set or hash_map
struct inthash
{
	unsigned int operator() (const int key) const
	{
        return hashfunc(key);
	}
};

/*struct int64hash
{
	unsigned int operator()(const uint64_t key) const
	{
        return hashfunc(key);
	}
};*/
//equal function used in hash_set or hash_map
struct eqstr
{
  bool operator()(const string s1, const string s2) const
  {
    return  (s1==s2);
  }
};

//trim a string
string trimStr(string str);

//change a string to lower case string
string tolower(string str);

//replace"\'" and "\"" into " " in the string
void replaceBadChar(string& str, string bad, char good);
void replaceBadChar(string& str, char bad, string good);

template <class T>
void setBit(T* array, int position)
{
        int unitsize=sizeof(T)*8;
        int start=position/unitsize;
        int offset=position%unitsize;
        T mask=1;
        mask<<=(unitsize-1-offset);
        array[start]|=mask;
}

uint64_t getOrthSeq(const vector<int>& orthlogs);

void getDistSeqSum(const vector<uint32_t>& distMatrix, 
		uint32_t& summary, string& matrixStr);

void getNodeSeq(const vector<int>& nodes, string& nodeSeq);

int difference(const vector<int>& vec1, const vector<int>& vec2);

void getBitString(const vector<bool>& bitarry, char str[]);

void getBoolArray(vector<unsigned short> nbarray, bool* boolarray);

bool isBitSet( unsigned short* array, int position);

float calScore(int deg_q, int deg_db, int conn_q, int conn_db, int miss);

bool readOneLine(FILE* f, string& str);//the '\n' is removed

int getOneSegment(string str, int start, char delim, string& seg);

void readItems(string str, char delim, vector<string>& items);

template <class T>
string toString(vector<T>&  array, char delim)
{
	if(array.empty()) return "";
	ostringstream opstr;
	opstr<<array[0];
	for(unsigned int i=1; i<array.size(); i++)
		opstr<<delim<<array[i];
	//opstr<<ends;
	string str=opstr.str();
	debug(1, str<<endl);
	return str;
}
#endif
